/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.admin;


import com.funambol.server.config.Configuration;
import com.funambol.framework.tools.beans.BeanException;
import com.funambol.framework.tools.beans.BeanInstantiationException;
import com.funambol.framework.tools.beans.BeanInitializationException;
import com.funambol.framework.tools.beans.BeanNotFoundException;

import com.funambol.email.exception.EmailConfigException;


/**
 * This class loads the parameter from the XML configuration file
 *
 * @version $Id: EmailConnectorConfig.java,v 1.9 2006/04/28 13:03:32 alexdeac Exp $
 */
public class EmailConnectorConfig {

    // --------------------------------------------------------------- CONSTANTS

    /**     */
    public final static String beanName = "email/email/Email.xml";

    // -------------------------------------------------------------- PROPERTIES

    /**
     * backend server
     */
    private String host;

    /**
     * protocol
     */
    private String protocol;

    /**
     * port
     */
    private int port;

    /**
     * ssl activation
     */
    private boolean ssl;

    /**
     * keystore path
     */
    private String keystorePath;

    /**
     * keystore passPhrase
     */
    private String keystorePassPhrase;

    /**
     * datasource
     */
    private String dataSource;

    /**
     * filter activation
     */
    private boolean filter;

    //------------------------------------------------------------- CONSTRUCTORS

    /**
     *
     */
    public EmailConnectorConfig() {
    }

    //----------------------------------------------------------- PUBLIC METHODS

    /**
     * @return EmailConnectorConfig
     * @throws EmailConfigException
     */
    public static EmailConnectorConfig getConfigInstance()
            throws EmailConfigException {

        EmailConnectorConfig emailConf;
        try {
            emailConf = (EmailConnectorConfig) Configuration.getConfiguration().getBeanInstanceByName(beanName);
        } catch (BeanInstantiationException bie) {
            throw new EmailConfigException(bie.getMessage());
        } catch (BeanInitializationException bie) {
            throw new EmailConfigException(bie.getMessage());
        } catch (BeanNotFoundException bie) {
            throw new EmailConfigException(bie.getMessage());
        } catch (BeanException be) {
            throw new EmailConfigException(be.getMessage());
        }

        return emailConf;

    }


    /**
     * @param host String
     */
    public void setHost(String host) {
        this.host = host;
    }

    /**
     * @return inHost String
     */
    public String getHost() {
        return this.host;
    }


    /**
     * @param ssl boolean
     */
    public void setSsl(boolean ssl) {
        this.ssl = ssl;
    }

    /**
     * @return ssl boolean
     */
    public boolean getSsl() {
        return this.ssl;
    }

    /**
     * @param keystorePath String
     */
    public void setKeystorePath(String keystorePath) {
        this.keystorePath = keystorePath;
    }

    /**
     * @return keystorePath String
     */
    public String getKeystorePath() {
        return this.keystorePath;
    }


    /**
     * @param protocol String
     */
    public void setProtocol(String protocol) {
        this.protocol = protocol;
    }

    /**
     * @return protocol String
     */
    public String getProtocol() {
        return this.protocol;
    }


    /**
     * @param port int
     */
    public void setPort(int port) {
        this.port = port;
    }

    /**
     * @return port int
     */
    public int getPort() {
        return this.port;
    }

    /**
     * @param dataSource String
     */
    public void setDataSource(String dataSource) {
        this.dataSource = dataSource;
    }

    /**
     * @return dataSource
     */
    public String getDataSource() {
        return this.dataSource;
    }


    /**
     * @param filter boolean
     */
    public void setFilter(boolean filter) {
        this.filter = filter;
    }

    /**
     * @return filter
     */
    public boolean getFilter() {
        return this.filter;
    }

    /**
     * @param keystorePassPhrase
     */
    public void setKeystorePassPhrase(String keystorePassPhrase) {
        this.keystorePassPhrase = keystorePassPhrase;
    }

    /**
     * @return keystorePassPhrase
     */
    public String getKeystorePassPhrase() {
        return keystorePassPhrase;
    }

}
