/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.engine.cache;

import java.io.Serializable;
import java.security.Principal;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.logging.*;

import com.funambol.framework.core.*;
import com.funambol.framework.engine.*;
import com.funambol.framework.engine.source.*;
import com.funambol.framework.security.*;
import com.funambol.framework.tools.beans.*;

import com.funambol.email.admin.EmailConnectorConfig;
import com.funambol.email.exception.EmailConfigException;
import com.funambol.email.util.Def;


/**
 * This class includes the common methods for the OpenSource CrcCachedSyncSource 
 * and the Enterprise CrcCachedSyncSource.
 *
 * @version $Id: CrcCachedSyncSourceHelper.java,v 1.1 2006/03/29 11:00:59 gbmiglia Exp $
 *
 */
public class CrcCachedSyncSourceHelper {

    // --------------------------------------------------------------- Constants

    private Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // ---------------------------------------------------------- Public Methods

    /**
     * BeginSync helper
     *
     * @param context The sync context that includes the principal,
     * sync mode and filter clause.
     * @throws SyncSourceException
     */
    /*
    public void beginSyncHelper(String sourceURI,
                                long principalId, 
                                int syncMode, 
                                CrcItemStateDAO crcItemStateDao,
                                CrcSyncItemInfo[] serverItems,
                                CrcSyncItemInfo[] localItems,
                                ArrayList newItemKeys,
                                ArrayList updatedItemKeys,
                                ArrayList deletedItemKeys)
        throws SyncSourceException {

        try {


            if (syncMode == AlertCode.SLOW){
                 
                crcItemStateDao.deleteLocalItems(sourceURI,principalId);
                crcItemStateDao.deleteSentPop(sourceURI,principalId);

            } else if (syncMode == AlertCode.REFRESH_FROM_SERVER){
                
                crcItemStateDao.deleteLocalItems(sourceURI,principalId);
                crcItemStateDao.deleteSentPop(sourceURI,principalId);

            } else if (syncMode == AlertCode.REFRESH_FROM_CLIENT) {
                
                // delete all info on the server
                crcItemStateDao.deleteLocalItems(sourceURI,principalId);
                crcItemStateDao.deleteSentPop(sourceURI,principalId);
                clearServerInfo();
                
            } else {
                
                serverItems = getAllSyncItemInfo();
                
                localItems  = crcItemStateDao.getLocalItems(sourceURI,
                                                            principalId);

                newItemKeys     = setNewItemKeys(localItems, serverItems);
                updatedItemKeys = setUpdatedItemKeys(localItems, serverItems);
                deletedItemKeys = setDeletedItemKeys(localItems, serverItems);
            }


        } catch (Exception e) {
            throw new SyncSourceException("Error accessing items: " +
                                           e.getMessage(), e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Crc Cached NEW");
            if (newItemKeys != null){
               log.finest("Crc Cached NEW num:" + newItemKeys.size());
            } else {
               log.finest("Crc Cached NO NEW ITEMS");
            }

            log.finest("Crc Cached UPDATED");
            if (updatedItemKeys != null){
               log.finest("Crc Cached UPDATED num:" + updatedItemKeys.size());
            } else {
               log.finest("Crc Cached NO UPDATED ITEMS");
            }

            log.finest("Crc Cached DELETED");
            if (deletedItemKeys != null){
               log.finest("Crc Cached DELETED num:" + deletedItemKeys.size());
            } else {
               log.finest("Crc Cached NO DELETED ITEMS");
            }
        }

    }
    */
    
    
    /**
     *
     *
     * @param syncItemKey SyncItemKey
     * @return char
     * @throws SyncSourceException
     */
    public static char getSyncItemStateFromId(SyncItemKey syncItemKey,
                                              int syncMode,
                                              ArrayList newItemKeys,
                                              ArrayList deletedItemKeys,
                                              ArrayList updatedItemKeys)
    throws SyncSourceException {

        char status       = SyncItemState.UNKNOWN;
        boolean isNew     = false;
        boolean isUpdated = false;
        boolean isDeleted = false;

        String id = syncItemKey.getKeyAsString();

        //
        if ((syncMode == AlertCode.SLOW) ||
            (syncMode == AlertCode.REFRESH_FROM_SERVER)) {
           return SyncItemState.NEW;
        }

        // search in new items list
        SyncItemKey tmp = null;
        for (int i=0; i<newItemKeys.size(); i++){
            
            tmp = (SyncItemKey)newItemKeys.get(i);
            
            if (tmp.getKeyAsString().equals(id)) {
                isNew  = true;
                status = SyncItemState.NEW;
                break;
            }
            
        }

        if (!isNew){

           // search in updated items list
           for (int i=0; i<updatedItemKeys.size(); i++){
               
              tmp = (SyncItemKey)updatedItemKeys.get(i);
              
              if (tmp.getKeyAsString().equals(id)) {
                 isUpdated = true;
                 status    = SyncItemState.UPDATED;
                 break;
              }
           }


           if (!isUpdated){
              // search in deleted items list
              for (int i=0; i<deletedItemKeys.size(); i++){
                  
                 tmp = (SyncItemKey)deletedItemKeys.get(i);
                 
                 if (tmp.getKeyAsString().equals(id)) {
                    isDeleted = true;
                    status    = SyncItemState.DELETED;
                    break;
                 }
              }

              if (!isDeleted){
                 status    = SyncItemState.SYNCHRONIZED;
              }

           }

        }

        return status;

    }

    /**
     *
     *
     * @param id String
     * @throws SyncSourceException
     */
    public static void insertSentItem(String id,
                                      String sourceURI,
                                      long principalId,
                                      CrcItemStateDAO crcItemStateDao)
        throws SyncSourceException {

        CrcSyncItemInfo[] sentItems = new CrcSyncItemInfo[1];

        CrcSyncItemInfo sentItem = new CrcSyncItemInfo();
        sentItem.setKey(new SyncItemKey(id));
        sentItem.setLastCrc(1);

        sentItems[0] = sentItem;

        try {
           crcItemStateDao.addLocalItems(sentItems, sourceURI, principalId);
        } catch (Exception e) {
            throw new SyncSourceException("Error caching sent items:" +
                                          e.getMessage(), e);
        }

    }


    /**
     * Creates new items id array
     *
     * @return new items id array
     */
    public static ArrayList setNewItemKeys(CrcSyncItemInfo[] localItems,
                                     CrcSyncItemInfo[] serverItems) {

        ArrayList  newRows         = null ;
        CrcSyncItemInfo serverItem  = null ;

        newRows = new ArrayList();

        int l = serverItems.length;

        for (int i = 0; i < l; i++) {

            serverItem = serverItems[i];

            if (!isItemInArray(localItems, serverItem)) {
                newRows.add(serverItem.getKey());
            }

        }
        return newRows;
    }

    /**
     * Creates deleted items id array
     *
     * @return deleted items id array
     */
    public static ArrayList setDeletedItemKeys(CrcSyncItemInfo[] localItems,
                                               CrcSyncItemInfo[] serverItems) {

        CrcSyncItemInfo localItem = null ;
        ArrayList deleteRows      = new ArrayList();

        for (int i=0, l = localItems.length; i < l; i++) {
           localItem = localItems[i];
           if (!isItemInArray(serverItems, localItem)) {
               deleteRows.add(localItem.getKey());
           }
        }

        return deleteRows;
    }

    /**
     * Creates updated items id array
     *
     * @return updated items id array
     */
    public static ArrayList setUpdatedItemKeys(CrcSyncItemInfo[] localItems,
                                               CrcSyncItemInfo[] serverItems) {

        ArrayList updateRows      = new ArrayList();

        CrcSyncItemInfo serverItem   = null ;
        CrcSyncItemInfo localItem    = null ;

        long serverItemLastCrc ;
        long localItemLastCrc  ;

        for (int i = 0, l = serverItems.length; i < l; i++) {

            serverItem = serverItems[i];
            localItem  = getItemById(localItems,
                                     serverItem.getKey().getKeyAsString());

            if (localItem != null) {
                 serverItemLastCrc   = serverItem.getLastCrc () ;
                 localItemLastCrc    = localItem.getLastCrc  () ;

                 if (serverItemLastCrc != localItemLastCrc) {
                    updateRows.add(serverItem.getKey());
                 }
             }
        }

        //return (SyncItemKey[])updateRows.toArray(new SyncItemKey[0]) ;
        return updateRows ;

    }
    
    /**
     * Checks if item is included in array of items
     *
     * @param items items array
     * @param item item to find
     * @return <p>true</p> if the item is included
     *         <p>false</p> if the item is not included
     **/
    public static boolean isItemInArray(CrcSyncItemInfo[] items,
                                  CrcSyncItemInfo   item) {

        return isIdInArray(items, item.getKey().getKeyAsString());

    }

    //---------------------------------------------------------- Private Methods

    /**
     * Checks if item id is included in ids from array of items
     *
     * @param items items array
     * @param id id to find
     * @return <p>true</p> if the item is included
     *         <p>false</p> if the item is not included
     **/
    private static boolean isIdInArray(CrcSyncItemInfo[] items,
                                String id) {
        boolean isFind = false;
        int l = items.length;
        for (int i = 0; i < l; i++) {
            if (items[i].getKey().getKeyAsString().equals(id)) {
                isFind = true;
                break;
            }
        }
        return isFind;
    }


    /**
     * Gets item by id
     *
     *
     * @param items CrcSyncItemInfo[]
     * @param id String
     * @return <p>item</p> if found
     *         <p>null</p> if not found
     */
    private static CrcSyncItemInfo getItemById(CrcSyncItemInfo[] items, 
                                               String id) {
        CrcSyncItemInfo item = null;
        
        for (int i = 0, l = items.length; i < l; i++) {
            if (items[i].getKey().getKeyAsString().equals(id)) {
                item = items[i];
                break;
            }
        }
        
        return item;
    }


}
