/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.engine.source;

import java.io.Serializable;
import java.io.ByteArrayInputStream;
import java.util.TimeZone;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.ArrayList;
import java.security.Principal;

import com.funambol.framework.engine.SyncItem;
import com.funambol.framework.security.Sync4jPrincipal;
import com.funambol.framework.server.Sync4jDevice;
import com.funambol.framework.server.store.PersistentStoreException;
import com.funambol.framework.server.store.PersistentStore;
import com.funambol.framework.engine.SyncItemImpl;
import com.funambol.framework.engine.source.*;
import com.funambol.framework.filter.FilterClause;
import com.funambol.framework.filter.LogicalClause;
import com.funambol.framework.filter.WhereClause;
import com.funambol.framework.filter.FieldClause;
import com.funambol.framework.filter.IDClause;
import com.funambol.framework.filter.Clause;
import com.funambol.framework.core.PropParam;
import com.funambol.framework.core.Property;
import com.funambol.server.config.Configuration;
import com.funambol.server.tools.IdSpaceGenerator;

import com.funambol.email.admin.EmailConnectorConfig;
import com.funambol.email.pdi.parser.XMLMailParser;
import com.funambol.email.pdi.converter.MailToXML;
import com.funambol.email.engine.cache.CrcCachedSyncSource;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.EmailConfigException;
import com.funambol.email.exception.EntityException;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.MailServerWrapperFactory;
import com.funambol.email.items.model.SendResult;
import com.funambol.email.items.manager.EntityManager;
import com.funambol.email.pdi.folder.Folder;
import com.funambol.email.pdi.parser.XMLFolderParser;
import com.funambol.email.pdi.converter.FolderToXML;
import com.funambol.email.pdi.mail.Email;
import com.funambol.email.items.model.DefaultFolder;
import com.funambol.email.items.model.EmailFilter;


/**
 * <p>This class implements common methods of
 * <pre>
 *  EmailIMAPSyncSource
 *  EmailPOPSyncSource
 * </pre>
 * <p/>
 * version $Id: EmailSyncSource.java,v 1.28 2006/05/15 17:35:13 gbmiglia Exp $
 */
public abstract class EmailSyncSource
        extends CrcCachedSyncSource
        implements FilterableSyncSource, Serializable {

    // --------------------------------------------------------------- Constants

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // -------------------------------------------------------------- Properties

    /**
     * Property for sync session
     */
    protected Sync4jDevice device = null;

    /**
     * Property for sync session
     */
    protected String deviceTimeZoneDescr = null;

    /**
     * Property for sync session
     */
    protected TimeZone deviceTimeZone = null;

    /**
     * Property for sync session
     */
    protected String deviceCharset = null;

    /**
     * Property Principal
     */
    protected Principal principal = null;

    /**
     * Property syncMode
     */
    protected int syncMode;

    /**
     * wrapper for the Mail Server
     */
    protected IMailServerWrapper mswf = null;

    /**
     *
     */
    protected EntityManager em = null;

    /**
     *
     */
    protected EmailFilter filter = new EmailFilter();

    /**
     *
     */
    protected static IdSpaceGenerator idSentSpace =
            new IdSpaceGenerator(Def.ID_COUNTER_SENT);

    /**
     *
     */
    protected static IdSpaceGenerator idFolderSpace =
            new IdSpaceGenerator(Def.ID_COUNTER_FOLDER);


    {
        this.setType("application/vnd.omads-email+xml,application/vnd.omads-folder+xml");
        this.setInfo(new SyncSourceInfo(
                new ContentType[]{
                        new ContentType("application/vnd.omads-email+xml", "1.2"),
                        new ContentType("application/vnd.omads-folder+xml", "1.2")
                }, 0));
    }


    /**
     * Should the channel be ssl?
     */
    protected boolean ssl = false;

    /**
     * Getter for property ssl.
     *
     * @return Value of property ssl.
     */
    public boolean isSSL() {
        return ssl;
    }

    /**
     * Setter for property ssl.
     *
     * @param ssl New value of property ssl.
     */
    public void setSSL(boolean ssl) {
        this.ssl = ssl;
    }

    /**
     * Property keystorePath
     */
    protected String keystorePath = "";

    /**
     * Getter for property keystorePath
     *
     * @return Value of property keystorePath
     */
    public String getKeystorePath() {
        return this.keystorePath;
    }

    /**
     * Setter for property keystorePath
     *
     * @param keystorePath String. New value of property keystorePath
     */
    public void setKeystorePath(String keystorePath) {
        this.keystorePath = keystorePath;
    }

    /**
     * Property keystorePassphrase
     */
    protected String keystorePassphrase = "";

    /**
     * Getter for property keystorePassphrase
     *
     * @return Value of property keystorePassphrase
     */
    public String getKeystorePassphrase() {
        return keystorePassphrase;
    }

    /**
     * Setter for keystorePassphrase
     *
     * @param keystorePassphrase String. New value of property keystorePassphrase
     */
    public void setKeystorePassphrase(String keystorePassphrase) {
        this.keystorePassphrase = keystorePassphrase;
    }

    /**
     * Should the channel be allMailboxActivation?
     */
    protected boolean allMailboxActivation = false;

    /**
     * Getter for property allMailboxActivation.
     *
     * @return Value of property allMailboxActivation.
     */
    public boolean isAllMailboxActivation() {
        return allMailboxActivation;
    }

    /**
     * Setter for property allMailboxActivation.
     *
     * @param allMailboxActivation New value of property allMailboxActivation.
     */
    public void setAllMailboxActivation(boolean allMailboxActivation) {
        this.allMailboxActivation = allMailboxActivation;
    }

    /**
     * Should the channel be inboxActivation?
     */
    protected boolean inboxActivation = true;

    /**
     * Getter for property inboxActivation.
     *
     * @return Value of property inboxActivation.
     */
    public boolean isInboxActivation() {
        return inboxActivation;
    }

    /**
     * Setter for property inboxActivation.
     *
     * @param inboxActivation New value of property inboxActivation.
     */
    public void setInboxActivation(boolean inboxActivation) {
        this.inboxActivation = inboxActivation;
    }


    /**
     * Should the channel be outboxActivation?
     */
    protected boolean outboxActivation = true;

    /**
     * Getter for property outboxActivation.
     *
     * @return Value of property outboxActivation.
     */
    public boolean isOutboxActivation() {
        return outboxActivation;
    }

    /**
     * Setter for property outboxActivation.
     *
     * @param outboxActivation New value of property outboxActivation.
     */
    public void setOutboxActivation(boolean outboxActivation) {
        this.outboxActivation = outboxActivation;
    }


    /**
     * Should the channel be sentActivation?
     */
    protected boolean sentActivation = true;

    /**
     * Getter for property sentActivation.
     *
     * @return Value of property sentActivation.
     */
    public boolean isSentActivation() {
        return sentActivation;
    }

    /**
     * Setter for property sentActivation.
     *
     * @param sentActivation New value of property sentActivation.
     */
    public void setSentActivation(boolean sentActivation) {
        this.sentActivation = sentActivation;
    }


    /**
     * Should the channel be draftActivation
     */
    protected boolean draftActivation = false;

    /**
     * Getter for property draftActivation.
     *
     * @return Value of property draftActivation.
     */
    public boolean isDraftActivation() {
        return draftActivation;
    }

    /**
     * Setter for property draftActivation.
     *
     * @param draftActivation New value of property draftActivation.
     */
    public void setDraftActivation(boolean draftActivation) {
        this.draftActivation = draftActivation;
    }

    /**
     * Should the channel be trashActivation
     */
    protected boolean trashActivation = false;

    /**
     * Getter for property trashActivation.
     *
     * @return Value of property trashActivation.
     */
    public boolean isTrashActivation() {
        return trashActivation;
    }

    /**
     * Setter for property trashActivation.
     *
     * @param trashActivation New value of property trashActivation.
     */
    public void setTrashActivation(boolean trashActivation) {
        this.trashActivation = trashActivation;
    }


    /**
     * Property Incoming Server
     */
    protected String inHost = "";

    /**
     * Getter for property Incoming Mail Server Host.
     *
     * @return Value of property Incoming Mail Server Host.
     */
    public String getInHost() {
        return inHost;
    }

    /**
     * Setter for property Incoming Mail Server.
     *
     * @param inHost String New value of property Incoming Mail Server.
     */
    public void setInHost(String inHost) {
        this.inHost = inHost;
    }

    /**
     * Property Incoming Server Port
     */
    protected String inPort = "";

    /**
     * Getter for property Incoming Mail Server Port.
     *
     * @return Value of property Incoming Mail Server Port.
     */
    public String getInPort() {
        return inPort;
    }

    /**
     * Setter for property Incoming Mail Server Port.
     *
     * @param inPort String New value of property Incoming Mail Server Port.
     */
    public void setInPort(String inPort) {
        this.inPort = inPort;
    }


    /**
     * Property Incoming Server
     */
    protected String outHost = "";

    /**
     * Getter for property Outgoing Mail Server Host.
     *
     * @return Value of property Outgoing Mail Server Host.
     */
    public String getOutHost() {
        return outHost;
    }

    /**
     * Setter for property Outgoing Mail Server.
     *
     * @param outHost String New value of property Outgoing Mail Server.
     */
    public void setOutHost(String outHost) {
        this.outHost = outHost;
    }

    /**
     * Property Incoming Server Port
     */
    protected String outPort = "25";

    /**
     * Getter for property Outgoing Mail Server Port.
     *
     * @return Value of property Incoming Mail Server Port.
     */
    public String getOutPort() {
        return outPort;
    }

    /**
     * Setter for property Outgoing Mail Server Port.
     *
     * @param outPort String New value of property Outgoing Mail Server Port.
     */
    public void setOutPort(String outPort) {
        this.outPort = outPort;
    }

    /**
     * Property Mail Server Server protocol
     */
    protected String protocol = "";

    /**
     * Getter for property Mail Server protocol.
     *
     * @return Value of property Mail Server protocol.
     */
    public String getProtocol() {
        return this.protocol;
    }

    /**
     * Setter for property Mail Server protocol.
     *
     * @param protocol String. New value of property Mail Server protocol.
     */
    public void setProtocol(String protocol) {
        this.protocol = protocol;
    }


    /**
     * Should the content be encoded?
     */
    protected DefaultFolder defaultFolder = new DefaultFolder(
            Def.FOLDER_INBOX_ENG,
            Def.FOLDER_OUTBOX_ENG,
            Def.FOLDER_SENT_ENG,
            Def.FOLDER_DRAFTS_ENG,
            Def.FOLDER_TRASH_ENG);

    /**
     * Getter for property defaultFolder.
     *
     * @return Value of property defaultFolder.
     */
    public DefaultFolder getDefaultFolder() {
        return this.defaultFolder;
    }

    /**
     * Setter for property defaultFolder.
     *
     * @param i String
     * @param o String
     * @param s String
     * @param d String
     * @param t String
     */
    public void setDefaultFolder(String i,
                                 String o,
                                 String s,
                                 String d,
                                 String t) {
        this.defaultFolder.setInboxName(i);
        this.defaultFolder.setInboxId(Def.FOLDER_INBOX_ID);
        this.defaultFolder.setOutboxName(o);
        this.defaultFolder.setOutboxId(Def.FOLDER_OUTBOX_ID);
        this.defaultFolder.setSentName(s);
        this.defaultFolder.setSentId(Def.FOLDER_SENT_ID);
        this.defaultFolder.setDraftName(d);
        this.defaultFolder.setDraftId(Def.FOLDER_DRAFTS_ID);
        this.defaultFolder.setTrashName(t);
        this.defaultFolder.setTrashId(Def.FOLDER_TRASH_ID);
    }

    // ------------------------------------------------------------ Public Methods

    /**
     * Begin Sync Method
     * <p/>
     * before starting sync the method set the filter
     * <p/>
     * main folder filter
     * <ul>
     * <li> T D S O I
     * <li> 0 0 0 0 1 -->  1   inbox
     * <li> 0 0 0 1 1 -->  3   inbox + outbox
     * <li> 0 0 1 1 1 -->  7   inbox + outbox + sent
     * <li> 0 1 1 1 1 -->  15  inbox + outbox + sent + draft
     * <li> 1 1 1 1 1 -->  31  inbox + outbox + sent + draft + trash
     * </ul>
     * <p/>
     * <p/>
     * time filter (timeFilter)
     * <ul>
     * <li> null              --> all
     * <li> "i.e. 20-10-2005" --> filter
     * </ul>
     * <p/>
     * <p/>
     * size filter (sizeFilter / numbytes)
     * <ul>
     * <li> 000001 --> 1   FILTER_SIZE_H
     * <li> 000011 --> 3   FILTER_SIZE_H_PERC
     * <li> 000101 --> 5   FILTER_SIZE_H_B
     * <li> 001111 --> 15  FILTER_SIZE_H_B_PERC
     * <li> 010101 --> 21  FILTER_SIZE_H_B_A
     * <li> 111111 --> 63  FILTER_SIZE_H_B_A_PERC
     * </ul>
     *
     * @param context SyncContext
     * @throws SyncSourceException
     */
    public void beginSync(SyncContext context)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("EmailSyncSource - Start beginSync [" + sourceURI + "]");
        }

        // get Connector parameters
        boolean filterActivation = false;
        try {
            filterActivation = EmailConnectorConfig.getConfigInstance().getFilter();
        } catch (EmailConfigException e) {
            log.severe("Error Getting Connector Parameters:: " + e);
            log.throwing(getClass().getName(), "beginSync", e);
        }
        // get filter clause
        FilterClause filterClause = context.getFilterClause();

        if (filterActivation) {
            if (filterClause != null) {
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("GET Filter clause!!");
                }
                // first clause is Logical
                LogicalClause logicalClause = filterClause.getClause();
                setFilter(logicalClause);
            } else {
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("NO Filter clause!!");
                }
            }
        } else {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("NO Filter activation!!");
            }
        }
        setFilterFolder(); // anyway set filter folder

        // set common properties
        this.principal = context.getPrincipal();
        this.syncMode = context.getSyncMode();

        long principalId;
        String username;
        String credentials;
        String password;

        principalId = ((Sync4jPrincipal) this.principal).getId();
        username = ((Sync4jPrincipal) this.principal).getUsername();
        credentials = ((Sync4jPrincipal) this.principal).getEncodedCredentials();
        password = Utility.getPassword(username, credentials);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("\nCOORDS " +
                    "\nprincipalId  " + principalId +
                    "\ninHost       " + this.inHost +
                    "\noutHost      " + this.outHost +
                    "\nprotocol     " + this.protocol +
                    "\nfilter       " + filterActivation +
                    "\nlogin        " + username +
                    "\nfolders      (" + defaultFolder.getInboxName() + "/" +
                    defaultFolder.getOutboxName() + "/" +
                    defaultFolder.getSentName() + "/" +
                    defaultFolder.getDraftName() + "/" +
                    defaultFolder.getTrashName() + ")" +
                    "\nFILTER\n" + this.filter);
        }

        // open Mail Server connection
        this.mswf = MailServerWrapperFactory.getMailServerWrapper(this.protocol);

        try {
            this.mswf.openConnection(outHost, outPort, inHost, inPort,
                    defaultFolder, username, password,
                    ssl, keystorePath, keystorePassphrase);

        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error Connecting Mail Server:" +
                    e.getMessage(), e);
        }

        // insert the 5 main folder if needed
        insertDefaultFolder();

        // insert folders in the local DB if needed. keep the DB in sync
        //insertCustomFolder();


        if (log.isLoggable(Level.FINEST)) {
            log.finest("call Cached SyncSource");
        }

        super.beginSync(context);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Set Device Parameters");
        }

        String deviceId = null;
        try {
            deviceId = context.getPrincipal().getDeviceId();
            device = getDevice(deviceId);

            String timezone = device.getTimeZone();
            if (device.getConvertDate()) {
                if (timezone != null && timezone.length() > 0) {
                    deviceTimeZoneDescr = timezone;
                    deviceTimeZone = TimeZone.getTimeZone(deviceTimeZoneDescr);
                }
            }
            deviceCharset = device.getCharset();

        } catch (PersistentStoreException ex) {
            throw new SyncSourceException("Error reading device '" + deviceId + "'", ex);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("EmailSyncSource - End beginSync [" + sourceURI + "]");
        }

    }


    /**
     * End Sync Method
     *
     * @throws SyncSourceException
     */
    public void endSync() throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("EmailSyncSource - start EndSync [" + sourceURI + "]");
        }

        // reset the cache
        super.endSync();

        // move from OUTBOX to SENT the sent mails
        try {
            storageFinalizer();
        } catch (EntityException e) {
            throw new SyncSourceException("Error handling sent item:" +
                    e.getMessage(), e);
        }
        
        // close Mail Server connection
        try {
            this.mswf.releaseConnection();
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error Closing Email:" +
                    e.getMessage(), e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("EmailSyncSource - end EndSync [" + sourceURI + "]");
        }

    }

    /**
     * Get Data from sync4j.foundation.pdi.mail.Email
     * converting the Contact object into an xml item
     *
     * @param mail Email
     * @return String
     * @throws EntityException
     */
    public String getXMLFromFoundationMail(Email mail)
            throws EntityException {

        String xml;
        try {
            MailToXML m2xml = new MailToXML(this.deviceTimeZone,
                    this.deviceCharset);
            xml = m2xml.convert(mail);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        xml = handleContentOuput(xml);

        return xml;
    }

    /**
     * Get Data from foundation.pdi.mail.Folder
     * converting the Folder object into an xml item
     *
     * @param folder Folder
     * @return String
     * @throws EntityException
     */
    public String getXMLFromFoundationFolder(Folder folder)
            throws EntityException {

        String xml;
        try {
            FolderToXML m2xml = new FolderToXML(this.deviceTimeZone,
                    this.deviceCharset);
            xml = m2xml.convert(folder);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        xml = handleContentOuput(xml);

        return xml;
    }
    
    // --------------------------------------------------------- Protected Methods

    /**
     * Returns true if the syncitem is a Mail,
     * false if the sycItem is a Folder
     *
     * @param content String
     * @return boolean
     */
    protected boolean isEmailFromContent(String content) {

        boolean isEmail = true;

        if (content != null) {
            int emailIndex = content.indexOf("<Email>");
            int folderIndex = content.indexOf("<Folder>");
            if (log.isLoggable(Level.FINEST)) {
                log.finest("isEmail - emailIndex [" + emailIndex +
                        "] - folderIndex [ " + folderIndex + "]");
            }
            if ((folderIndex >= 0) && (emailIndex < 0)) {
                isEmail = false;
            }
        }

        return isEmail;
    }

    /**
     * Returns true if the syncitem is a Mail
     * false if the sycItem is a Folder
     * <p/>
     * the key is composed by: parentId/objId (mailid or folderid)
     * The key is created by the connector and it always
     * creates the syncItemKey using parentId and mailId
     *
     * @param parentId String
     * @param objectId String
     * @return boolean
     * @throws SyncSourceException
     */
    protected boolean isEmailFromKey(String parentId, String objectId)
            throws SyncSourceException {

        boolean isEmail;

        try {
            this.em = getEntityManager();
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting email: " +
                    e.getMessage(), e);
        }

        long pID = ((Sync4jPrincipal) this.principal).getId();
        try {
            isEmail = this.em.isEmail(parentId, objectId, this.sourceURI, pID);
        } catch (EntityException e) {
            throw new SyncSourceException("Error removing items", e);
        }

        return isEmail;
    }


    /**
     * insert the default folder if needed
     *
     * @throws SyncSourceException
     */
    protected void insertDefaultFolder() throws SyncSourceException {
        try {
            this.em = getEntityManager();
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting email: " +
                    e.getMessage(), e);
        }
        long pID = ((Sync4jPrincipal) this.principal).getId();
        try {
            this.em.insertDefaultFolder(this.defaultFolder,
                    this.sourceURI,
                    pID);
        } catch (EntityException e) {
            throw new SyncSourceException("Error removing items", e);
        }

    }

    /**
     * insert the default folder if needed
     *
     * @throws SyncSourceException
     */
    protected void insertCustomFolder() throws SyncSourceException {

        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {

            this.em = getEntityManager();

            this.em.insertCustomFolder(this.mswf,
                    this.allMailboxActivation,
                    idFolderSpace,
                    this.sourceURI,
                    pID);

        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting email: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error removing items", e);
        }

    }

    /**
     * Return the device with the given deviceId
     *
     * @param deviceId String
     * @return Sync4jDevice
     * @throws PersistentStoreException
     */
    protected Sync4jDevice getDevice(String deviceId) throws PersistentStoreException {
        Sync4jDevice device = new Sync4jDevice(deviceId);
        PersistentStore store = Configuration.getConfiguration().getStore();
        store.read(device);
        return device;
    }


    /**
     * Get Data from XML message converting the xml
     * item into a Email object
     *
     * @param content String
     * @return Email
     * @throws EntityException
     */
    public Email getFoundationMailFromXML(String content)
            throws EntityException {

        content = handleContentInput(content);

        ByteArrayInputStream buffer;
        XMLMailParser parser;
        Email mail;
        try {
            mail = new Email();
            buffer = new ByteArrayInputStream(content.getBytes());

            if ((content.getBytes()).length > 0) {
                parser = new XMLMailParser(buffer);
                mail = parser.parse();
            }
        } catch (Exception e) {
            throw new EntityException(e.toString());
        }
        return mail;
    }

    /**
     * Get Data from XML message converting the xml
     * item into a Folder object
     *
     * @param content String
     * @return Folder
     * @throws EntityException
     */
    public Folder getFoundationFolderFromXML(String content)
            throws EntityException {

        content = handleContentInput(content);

        ByteArrayInputStream buffer;
        XMLFolderParser parser;
        Folder folder;
        try {
            folder = new Folder();
            buffer = new ByteArrayInputStream(content.getBytes());

            if ((content.getBytes()).length > 0) {
                parser = new XMLFolderParser(buffer);
                folder = parser.parse();
            }
        } catch (Exception e) {
            throw new EntityException(e.toString());
        }
        return folder;
    }


    /**
     * @param email  Email
     * @param status char
     * @return SyncItem
     * @throws EntityException
     */
    protected SyncItem createSyncItem(Email email, char status)
            throws EntityException {

        SyncItem syncItem;
        String stream;

        stream = getXMLFromFoundationMail(email);

        String uid = (String) email.getUID().getPropertyValue();
        String parentId = (String) email.getParentId().getPropertyValue();

        try {
            syncItem = new SyncItemImpl(this, uid, parentId, status);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        syncItem.setType(Def.TYPE_EMAIL);

        syncItem.setContent(stream.getBytes());

        return syncItem;
    }

    /**
     * @param folder Folder
     * @param status char
     * @return SyncItem
     * @throws EntityException
     */
    protected SyncItem createSyncItem(Folder folder, char status)
            throws EntityException {

        SyncItem syncItem;
        String stream;

        stream = getXMLFromFoundationFolder(folder);

        String uid = (String) folder.getUID().getPropertyValue();
        String parentId = (String) folder.getParentId().getPropertyValue();

        try {
            syncItem = new SyncItemImpl(this, uid, parentId, status);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        syncItem.setType(Def.TYPE_FOLDER);

        syncItem.setContent(stream.getBytes());

        return syncItem;
    }

    //------------------------------------------------------------ PRIVATE METHODS

    /**
     * Create EntityManager istance (if not already exist)
     *
     * @return EntityManager
     * @throws EmailAccessException
     */
    private EntityManager getEntityManager() throws EmailAccessException {
        if (this.em == null) {
            this.em = new EntityManager(this.mswf, this.jndiDataSourceName);
        }
        return this.em;
    }

    /**
     * this method handles the content in a SyncItem
     * It's used in input direction
     *
     * at the moment it converts the end of the internal CDATA
     *
     * @param content String
     * @return String
     */
    private String handleContentInput(String content) {
        
        /*
         * tmp: the encrypt mail contains the external CDATA and \n
         */
        content = content.replaceAll("\\[\n","\\[");
        //content = content.replaceAll("]]&gt;\n","]]&gt;");
        //content = content.replaceAll(">\n", ">");
                
        if (content.startsWith("<![CDATA[<Email>")){
           int start = content.indexOf("<![CDATA[");
           int stop  =  content.indexOf("</Email>");
           content   = content.substring(start + 9, stop + 8);            
        }
        
        if (content.startsWith("<![CDATA[<Folder>")){
           int start = content.indexOf("<![CDATA[");
           int stop  =  content.indexOf("</Folder>");
           content = content.substring(start + 9, stop + 9);
        }        
        
        // 
        content = content.replaceAll("]]&gt;", "]]>");
        
        return content;
        
    }

    /**
     * this method handles the content in a SyncItem
     * It's used in output direction
     *
     * at the moment the output it's not modified
     *
     * @param stream String
     * @return String
     */
    private String handleContentOuput(String stream) {
        return stream;
    }


    /**
     * set the filter
     * <p/>
     * todo should be a recursive algorithm.
     * At the moment I do a fix parser (only 2 level)
     *
     * @param logicalClause LogicalClause
     */
    private void setFilter(LogicalClause logicalClause) {

        Clause[] operands = logicalClause.getOperands();

        for (int i = 0; i < operands.length; i++) {
            if (operands[i] instanceof IDClause) {
                IDClause tmp = (IDClause) operands[i];
                setFilterID(tmp);
            } else if (operands[i] instanceof WhereClause) {
                WhereClause tmp = (WhereClause) operands[i];
                setFilterTime(tmp);
            } else if (operands[i] instanceof FieldClause) {
                FieldClause tmp = (FieldClause) operands[i];
                setFilterSize(tmp);
            } else if (operands[i] instanceof LogicalClause) {
                LogicalClause tmp = (LogicalClause) operands[i];
                setFilter(tmp);
            }
        }
    }

    /**
     * set the time filter
     *
     * @param clause WhereClause
     */
    private void setFilterTime(WhereClause clause) {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setting setTimeFilter ");
        }

        String property = clause.getProperty();
        String operator = clause.getOperator();

        if (property.equalsIgnoreCase(Def.FILTER_TIME_FIELD)) {

            String[] values = clause.getValues();
            String value = values[0];

            if (log.isLoggable(Level.FINEST)) {
                log.finest("operator " + operator);
                log.finest("property " + property);
                log.finest("value    " + value);
            }

            if (value != null) {
                if (value.equalsIgnoreCase("ALL")) {
                    this.filter.setTime(null);
                } else {
                    this.filter.setTime(Utility.UtcToDate(value));
                    this.filter.setTimeClause(operator);
                }
            }
        }
    }


    /**
     * set the time filter
     *
     * @param icIdFilter IDClause
     */
    private void setFilterID(IDClause icIdFilter) {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setting setIdFilter ");
        }

        String property = icIdFilter.getProperty();
        String operator = icIdFilter.getOperator();

        String[] values = icIdFilter.getValues();
        String value = values[0];

        if (log.isLoggable(Level.FINEST)) {
            log.finest("operator " + operator);
            log.finest("property " + property);
            log.finest("value    " + value);
        }

        this.filter.setId(value);
    }

    /**
     * set the time filter
     */
    private void setFilterFolder() {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setting setFolderFilter ");
        }

        int f = 0;

        if (inboxActivation) {
            f = f | Def.FILTER_FOLDER_I;
        }
        if (outboxActivation) {
            f = f | Def.FILTER_FOLDER_IO;
        }
        if (sentActivation) {
            f = f | Def.FILTER_FOLDER_IOS;
        }
        if (draftActivation) {
            f = f | Def.FILTER_FOLDER_IOSD;
        }
        if (trashActivation) {
            f = f | Def.FILTER_FOLDER_IOSDT;
        }

        if (f == 0) {
            this.filter.setFolder(Def.FILTER_FOLDER_IOSD);
        } else {
            this.filter.setFolder(f);
        }

    }


    /**
     *
     * set the time filter
     *
     * @param fcSizeFilter FieldClause
     */
    private void setFilterSize(FieldClause fcSizeFilter) {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("setting setSizeFilter ");
        }

        Property property = fcSizeFilter.getProperty();

        String moduleName;
        ArrayList types;
        int maxSize;

        moduleName = property.getPropName();
        types = property.getPropParams();
        maxSize = property.getMaxSize();

        String typeName;
        int tempSizeFilter = 0;

        //  boolean isHeaderFieldSet = false;
        //  boolean isBodyFieldSet   = false;
        boolean isAttachFieldSet = false;

        if (moduleName.equalsIgnoreCase(Def.FILTER_SIZE_LABEL_HEADER)) {
            if (types.size() == 0) {
                tempSizeFilter = tempSizeFilter | Def.FILTER_SIZE_H;
            } else {
                for (int i = 0; i < types.size(); i++) {
                    typeName = ((PropParam) types.get(i)).getParamName();
                    if (!isAttachFieldSet) {
                        if (typeName.equalsIgnoreCase(Def.FILTER_SIZE_LABEL_BODY)) {
                            tempSizeFilter = tempSizeFilter | Def.FILTER_SIZE_H_B;
                            if (maxSize > 0) {
                                tempSizeFilter = tempSizeFilter | Def.FILTER_SIZE_H_B_PERC;
                                this.filter.setNumBytes(maxSize);
                            }
                        }
                    }
                    if (typeName.equalsIgnoreCase(Def.FILTER_SIZE_LABEL_ATTACH)) {
                        tempSizeFilter = tempSizeFilter | Def.FILTER_SIZE_H_B_A;
                        if (maxSize > 0) {
                            tempSizeFilter = tempSizeFilter | Def.FILTER_SIZE_H_B_A_PERC;
                            this.filter.setNumBytes(maxSize);
                        }
                        isAttachFieldSet = true;
                    }
                }
            }
        }

        if (tempSizeFilter == 0) {
            this.filter.setSize(1);
        } else {
            this.filter.setSize(tempSizeFilter);
     }

  }
    
  /**
   *
   * This method set the Storage (Mail Server for IMAP-like connector, 
   * DB for POP3-like connector ) in order to allow the server to send the
   * right commands in the OUTBOX/SENT folder synchronization procedure
   *
   * The OUTBOX/SENT folder synchronization procedure is composed by two steps:
   * in the first step the system saves the mail in the outbox folder, sends
   * the mail and the move the mail in the sent folder. in the second step
   * the system sends the the client the DELETE command for the document in the 
   * client outbox folder and the ADD command for the document in the 
   * client sent folder.
   *
   * if the email is not sent (there's an error in send procedure) we 
   * have to delete the raw in the sentpop table (POP) or outbox folder (IMAP) 
   * and in the cache table. 
   * Moreover the server doesn't update the mapping table.
   *
   * for a POP3-like settings:
   * at the end of the first sync the connector inserts 
   * a row in the cache table.
   *
   * in the second sync the cache system finds an id in the cache
   * for the outbox document but not the related document in the
   * Sentpop Table (DB).
   * Then the server sends a DELETE command for the oubox document;
   * morover the cache system finds a document in the Sentpop 
   * Table related to the sent document.
   * Then the server sends a ADD command for the client sent folder
   *
   * <p>
   * for a IMAP-like settings:
   * at the end of the first sync the connector moves message 
   * from Outbox folder to Sent folder
   * in the second sync the cache system finds an id in the cache
   * for the outbox document but nothing in the Outbox folder.
   * then the server sends a DELETE command for the client oubox 
   * folder;
   * morover the cache system finds a document in the server Sent 
   * folder but nothing in the cache.
   * Then the server sends a ADD command for the client sent 
   * folder;
   *
   *
   *
   * @throws EntityException
   *
   */
  private void storageFinalizer() throws EntityException {
      
        SendResult sr;
        String     GUID;
        String     LUID;
        String     FMID;
        boolean    isSent;
        boolean    isPOP;

        int sentLen = itemsSent.size();
        if (log.isLoggable(Level.FINEST)) {
            log.finest("EmailSyncSource - sent mail to move: " + sentLen);
        }

        long pID = ((Sync4jPrincipal) this.principal).getId();
        
        
        for (int i = 0; i < sentLen; i++) {

            sr = (SendResult) itemsSent.get(i);

            GUID   = sr.getGUID();
            LUID   = sr.getLUID();
            isSent = sr.isSent();
            isPOP  = sr.isPOP();

            if (log.isLoggable(Level.FINEST)) {
                log.finest(" sent mail guid:" + GUID + " luid: " + LUID +
                           " sent: " + isSent + " isPop: " + isPOP);
            }
            
            try {
                this.em = this.getEntityManager();
            } catch (Exception e) {
                throw new EntityException(e);
            }

            if (isPOP) { // POP3 protocol (uses the local DB)
                try {
                   if (isSent && GUID != null) {
                     FMID = Utility.getIdAfterSeparator(GUID);
                     this.em.moveSentItem(FMID, this.mswf, this.sourceURI, pID);
                   } else {
                     this.em.removePopOutboxItem(GUID, this.sourceURI, pID);
                   }
                } catch (Exception e) {
                    throw new EntityException(e);
                }
            } else { // IMAP protocol (uses the Mail Server)
                try {
                   if (isSent && GUID != null) {
                     FMID = Utility.getIdAfterSeparator(GUID);
                     this.em.moveSentItem(FMID, this.mswf, this.sourceURI, pID);
                   } else {
                     this.em.removeImapOutboxItem(GUID, this.mswf, this.sourceURI, pID);
                   }
                } catch (Exception e) {
                   throw new EntityException(e);
                }
            }
        }       
  }
    
  

}
