/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.inboxlistener.engine;


import java.util.List;
import java.util.ArrayList;
import java.util.logging.Logger;
import java.util.logging.Level;

import com.funambol.email.util.Def;


/**
 *
 *
 * @version $Id: ListenerRegistry.java,v 1.2 2006/03/14 16:30:16 gbmiglia Exp $
 */
public class ListenerRegistry extends Thread {

  //------------------------------------------------------------------ CONSTANTS

  private Logger log = Logger.getLogger(Def.LOGGER_NAME);

  //----------------------------------------------------------------- PROPERTIES

  /** list wtih all account to monitor  */
  private List accountList = new ArrayList() ;


  /**   */
  private DBAccount dba = null;

  /**   */
  private MSAccount msa = null;

  /**   */
  private WSAccount wsa = null;

  /**   */
  private String listenerId = null;

  /**   */
  private boolean startupNotification = false;

  /**   */
  private ListenerRegistryDAO lrdao = null;

  /** to stop the thread  */
  private boolean go = true ;


  //--------------------------------------------------------------- CONSTRUCTORS

  /**
   *
   */
  public ListenerRegistry(String listenerId,
                          boolean startupNotification,
                          DBAccount dba,
                          MSAccount msa,
                          WSAccount wsa) {
     this.listenerId          = listenerId;
     this.startupNotification = startupNotification;
     this.dba                 = dba;
     this.msa                 = msa;
     this.wsa                 = wsa;
     this.lrdao               = new ListenerRegistryDAO(dba);
  }


  //------------------------------------------------------------- PUBLIC METHODS

  /**
   *
   *
   */
  public void run() {

     if (log.isLoggable(Level.FINEST)) {
        log.finest("Run ListenerRegistry " );
     }

     while (go) {

        if (log.isLoggable(Level.FINEST)) {
           log.finest("Read Accounts");
        }
        // get list
        List accountListNew = null;
        try {
           accountListNew = lrdao.getAccounts(listenerId);
        } catch (Exception e){
           e.printStackTrace();
        }
        List accountListOld = this.accountList;
        if (accountListNew != null){
           startMailboxListener(accountListNew,accountListOld);
        }


        // sleep for X secs
        try{
           sleep(dba.getReloadingTime());
        } catch (Exception e){
           e.printStackTrace();
        }

     }

  }

  /**
   *
   */
  public void stopThread(){
     if (log.isLoggable(Level.FINEST)) {
        log.finest("Shutting down ListenerRegistry" );
     }
     this.go = false;
  }


  //------------------------------------------------------------ PRIVATE METHODS

  /**
   * @todo understand How stop the thread
   *
   */
  private void startMailboxListener(List accountListNew,
                                    List accountListOld){

      // remove listener for a deleted user
      String userOld = null;
      for (int i=0; i<accountListOld.size(); i++){
          userOld = ((MailboxAccount) accountListOld.get(i)).getUser();
          if (toRemove(userOld,accountListNew)){
             if (log.isLoggable(Level.FINEST)) {
                log.finest("Removing Listener for User " + userOld);
             }
             // stop mailbox listener
             MailboxListener ml = getListenerToStop(userOld);
             //ml.stopThread();
          }
      }

      // add listener for a new user
      String         userNew = null;
      MailboxAccount maNew   = null;
      for (int i=0; i<accountListNew.size(); i++){
          maNew   = (MailboxAccount) accountListNew.get(i);
          userNew = maNew.getUser();
          if (toAdd(userNew,accountListOld)){
             if (log.isLoggable(Level.FINEST)) {
                log.finest("Adding Listener for User " + userNew );
             }
             // start mailbox listener
             MailboxListener ml = new MailboxListener(this.startupNotification,
                                                      maNew,
                                                      msa,
                                                      wsa);
             ml.setName(userNew);
             ml.start();
          }
      }

      // replace the new account list
      accountList.clear();
      for (int i=0; i<accountListNew.size(); i++){
         maNew = (MailboxAccount) accountListNew.get(i);
         accountList.add(maNew);
      }


  }


  /**
   *
   * checks if the username is already present in the
   * account list to monitor.
   *
   * to remove
   * ---------
   * user is the Old user
   * accounts is the New List
   * if the username is not in the list it retrun true, the
   * listener on the account must be deregistered
   *
   *
   */
  private boolean toRemove(String userOld, List accountsNew){
     String         userNew           = null;
     boolean        isAlreadyRegister = false;
     boolean        toRemove          = true;

     for (int i=0; i<accountsNew.size(); i++){
        userNew = ((MailboxAccount) accountsNew.get(i)).getUser();
        if (userNew.equals(userOld)){
           isAlreadyRegister = true;
           break;
        }
     }

     if (isAlreadyRegister){
        toRemove = false;
     }

     return toRemove;
  }

  /**
   *
   * checks if the username is already present in the
   * account list to monitor.
   *
   *
   * to add
   * ---------
   * user is the New user
   * accounts is the Old List
   * if the username is not in the list it returns true,
   * the listener must be registered
   *
   *
   */
  private boolean toAdd(String userNew, List accountsOld){
     String         userOld           = null;
     boolean        isAlreadyRegister = false;
     boolean        toAdd             = true;

     for (int i=0; i<accountsOld.size(); i++){
        userOld = ((MailboxAccount) accountsOld.get(i)).getUser();
        if (userOld.equals(userNew)){
           isAlreadyRegister = true;
           break;
        }
     }

     if (isAlreadyRegister){
        toAdd = false;
     }
     return toAdd;
  }



  /**
   * greturns the listener to stop
   */
  private MailboxListener getListenerToStop(String user){

     MailboxListener mlToStop = null;
     Thread tobj              = null;
     String tname             = null;
     int activeCount = Thread.activeCount();
     ThreadGroup tg = Thread.currentThread().getThreadGroup();
     Thread[] list = new Thread[activeCount];
     tg.enumerate(list);
     for (int i=0; i<list.length; i++){
         tobj = (Thread)list[i];
         if (tobj != null){
            tname = tobj.getName();
            if ( tname.startsWith(user) ){
               break;
            }
         }
     }

     return mlToStop;
  }

}
