/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.items.dao;


import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.ArrayList;
import javax.mail.Message;
import javax.mail.Flags;
import javax.mail.MessagingException;
import com.sun.mail.imap.IMAPFolder;

import com.funambol.email.exception.EntityException;
import com.funambol.email.util.*;
import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;
import java.sql.PreparedStatement;
import com.funambol.framework.tools.DBTools;
import java.sql.Connection;
import com.funambol.server.tools.IdSpaceGenerator;
import java.sql.ResultSet;
import com.funambol.email.items.model.ItemFolder;
import com.sun.mail.pop3.POP3Folder;


/**
 * This class implements methods to access data
 * in Mail Server datastore.
 *
 * @version $Id: EntityDAO.java,v 1.13 2006/05/11 08:08:41 gbmiglia Exp $
 */
public class EntityDAO {


    // ------------------------------------------------------------ Private data

    protected Logger  log = Logger.getLogger(Def.LOGGER_NAME);

    /**     */
    protected String jndiDataSourceName = null;

    /**     */
    protected DataSource dataSource;

    /**
     * folders queries
     */
    String INSERT_FOLDER = "insert into fnbl_email_folder " +
                       "(guid, parentid, source_uri, principal, path) values " +
                       " (?,?,?,?,?)";

    String DELETE_FOLDER = "delete from fnbl_email_folder " +
                         " where guid = ? and source_uri = ? and principal = ?";

    String SELECT_FOLDER = "select guid, parentid, path from fnbl_email_folder " +
                         " where guid = ? and source_uri = ? and principal = ?";

    String SELECT_FULLPATH = "select path from fnbl_email_folder " +
                         " where guid = ? and source_uri = ? and principal = ?";

    String SELECT_FULLPATH_FROM_FID = "select path from fnbl_email_folder " +
                      " where guid like ? and source_uri = ? and principal = ?";

    String SELECT_FOLDERID = "select guid, parentid from fnbl_email_folder " +
                         " where path = ? and source_uri = ? and principal = ?";

    String SELECT_ALL_FOLDER = "select guid, parentid, path from fnbl_email_folder " +
                               " where source_uri = ? and principal = ?";

    /**
     * sent items queries
     */
    String INSERT_SENT = "insert into fnbl_email_sentpop " +
                         " (id, source_uri, principal, mail) values (?,?,?,?)";

    String UPDATE_SENT = "update fnbl_email_sentpop set id = ? " +
                         " where id = ? and source_uri = ? and principal = ?";

    String DELETE_SENT = "delete from fnbl_email_sentpop " +
                         " where id = ? and source_uri = ? and principal = ?";

    String DELETE_UNSENT = "delete from fnbl_email_cache " +
                           " where id = ? and source_uri = ? and principal = ? ";

    String SELECT_SENT = "select mail from fnbl_email_sentpop " +
                         " where id = ? and source_uri = ? and principal = ?";

    String SELECT_ALL_SENT = "select id from fnbl_email_sentpop " +
                             " where source_uri = ? and principal = ?";

    String SELECT_TWIN_SENT = "select id from fnbl_email_sentpop " +
                      " where mail like ? and source_uri = ? and principal = ?";


    // -------------------------------------------------------------- Properties


    /**
     *
     */
    public EntityDAO(){
    }

    /**
     *
     * @param jndiDataSourceName String
     */
    public EntityDAO(String jndiDataSourceName){
        this.jndiDataSourceName = jndiDataSourceName;
    }


    // -----------------------------------------------------------Public Methods


    /**
     *
     *
     *
     * @param sourceFolder IMAPFolder
     * @param targetFolder IMAPFolder
     * @param uid long
     * @throws EntityException
     */
    public void moveImapItem(IMAPFolder sourceFolder,
                             IMAPFolder targetFolder,
                             long uid)
      throws EntityException {

        try {
          Message msgToDelete    = sourceFolder.getMessageByUID(uid);
          Message[] msgsToDelete = new Message[1];
          msgsToDelete[0]        = msgToDelete;
	  targetFolder.appendMessages(msgsToDelete);
          sourceFolder.setFlags(msgsToDelete, new Flags(Flags.Flag.DELETED), true);
        }catch (MessagingException e){
            throw new EntityException(e);
        }

    }


    /**
     *
     *
     *
     * @param FMID String
     * @param source_uri String
     * @param principalId String
     * @throws EntityException
     */
    public void movePopItem(String FMID,
                            String source_uri,
                            long principalId)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start movePopItem FMID " + FMID);
        }
        Connection connection = null;
        PreparedStatement  ps = null ;
        try {
           lookupDataSource();
           connection = dataSource.getConnection();
           ps = connection.prepareStatement(UPDATE_SENT);
           ps.setString (1, "S/"+FMID ) ;
           ps.setString (2, "O/"+FMID ) ;
           ps.setString (3, source_uri ) ;
           ps.setLong   (4, principalId ) ;
           ps.executeUpdate();
           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }
        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO end movePopItem ");
        }

    }

    /**
     *
     * insert a custom folder
     *
     * @param folder IMAPFolder
     * @param defFullpath String
     * @param idFolderSpace IdSpaceGenerator
     * @param source_uri String
     * @param principalId long
     * @throws EntityException
     */
    public void insertCustomFolder(IMAPFolder folder, 
                                   String defFullpath,
                                   IdSpaceGenerator idFolderSpace,
                                   String source_uri, 
                                   long principalId)
       throws EntityException {

        String[] allFolder = null;

        // check all subfolders and insert if nedded
        try {
            insertSubFolder(folder, idFolderSpace, source_uri, principalId);
        } catch (MessagingException me){
            throw new EntityException(me);
        }


    }
    

    /**
     *
     * gets the folder fullpath from the local DB
     *
     *
     * @param GUID folder id
     * @param source_uri String
     * @param principalId long
     * @return String
     * @throws EntityException
     */
    public String getFullPathFromGUID(String GUID,
                                      String source_uri,
                                      long principalId)
      throws EntityException {

        String fullpath       = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;

        try {

           lookupDataSource();
           connection = dataSource.getConnection();
           ps = connection.prepareStatement(SELECT_FULLPATH);

           ps.setString (1, GUID        ) ;
           ps.setString (2, source_uri  ) ;
           ps.setLong   (3, principalId ) ;

           rs = ps.executeQuery();
           while(rs.next()) {
               fullpath = rs.getString(1);
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return fullpath;

    }

   
   /**
    *
    * gets the folder fullpath from the local DB
    * In the local db the GUID is composed by:
    * parentId + separator + objectId
    * This method look up the item with the 
    * GUID = '%/FID'
    * 
    *
    *
    * @param FID folder id
    * @param source_uri String
    * @param principalId long
    * @return String
    * @throws EntityException
    */
    public String getFullPathFromFID(String FID,
                                      String source_uri,
                                      long principalId)
      throws EntityException {

        String fullpath       = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;


        try {

           lookupDataSource();
           connection = dataSource.getConnection();
           ps = connection.prepareStatement(SELECT_FULLPATH_FROM_FID);

           ps.setString (1, "%" + Def.SEPARATOR + FID ) ;
           ps.setString (2, source_uri                ) ;
           ps.setLong   (3, principalId               ) ;

           rs = ps.executeQuery();
           while(rs.next()) {
               fullpath = rs.getString(1);
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return fullpath;

    }
    
    
    /**
     *
     * gets the Folder id from the local DB
     *
     *
     * @param fullpath folder full path
     * @param source_uri String
     * @param principalId long
     * @return String
     * @throws EntityException
     */
    public String getGUIDFromFullPath(String fullpath,
                                      String source_uri,
                                      long principalId)
      throws EntityException {

        String GUID           = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;

        try {

           lookupDataSource();
           connection = dataSource.getConnection();
           ps = connection.prepareStatement(SELECT_FOLDERID);

           ps.setString (1, fullpath    ) ;
           ps.setString (2, source_uri  ) ;
           ps.setLong   (3, principalId ) ;

           rs = ps.executeQuery();
           while(rs.next()) {
               GUID     = rs.getString(1);
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return GUID;

    }

    /**
     *
     * serch a folder using the GUID in the local DB
     *
     *
     * @param GUID folder id
     * @param source_uri String
     * @param principalId long
     * @return ItemFolder
     * @throws EntityException
     */
    public ItemFolder getFolderFromUID(String GUID,
                                       String source_uri,
                                       long principalId)
       throws EntityException {

        ItemFolder eisFolder   = null;
        String     fullpath    = null;
        Connection connection  = null;
        PreparedStatement ps   = null;
        ResultSet rs           = null;

        try {
           lookupDataSource();
           connection = dataSource.getConnection();
           ps = connection.prepareStatement(SELECT_FOLDER);
           ps.setString (1, GUID ) ;
           ps.setString (2, source_uri ) ;
           ps.setLong   (3, principalId ) ;
           rs = ps.executeQuery();

           String FID  = null;
           while(rs.next()) {
               GUID      = rs.getString(1);
               FID       = rs.getString(2);
               fullpath  = rs.getString(3);
           }

           if (fullpath != null) {
               
               String name = Utility.getFolderNameFromFullpath(fullpath);

               eisFolder = new ItemFolder(GUID, name, FID, null,
                                          null, null, null, null);
           }

           ps.close() ;
           ps = null  ;

        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return eisFolder;
    }


  /**
   *
   *
   *
   * @param fullPath folder full path
   * @param parentId parent folder id
   * @param idFolderSpace IdSpaceGenerator
   * @param source_uri String
   * @param principalId long
   * @return String
   * @throws EntityException
   */
  public String insertFolder(String fullPath,
                             String parentId,
                             IdSpaceGenerator idFolderSpace,
                             String source_uri,
                             long principalId)
    throws EntityException {

      Connection connection = null;
      PreparedStatement  ps = null;
      String GUID           = null;
      String FID            = null;

      try {

         lookupDataSource();
         connection = dataSource.getConnection();
         ps = connection.prepareStatement(INSERT_FOLDER);

         GUID = Utility.getGUID(parentId, idFolderSpace.next());
         
         FID = parentId;

         ps.setString(1, GUID);
         ps.setString(2, FID);
         ps.setString(3, source_uri);
         ps.setLong(4, principalId);
         ps.setString(5, fullPath);

         ps.executeUpdate();

         ps.close() ;
         ps = null  ;

      } catch (Exception e) {
          throw new EntityException(e);
      } finally {
          DBTools.close(connection, ps, null);
      }

      return GUID;

  }
  
  
  /**
   *
   *
   * @param GUID  the Global Unique ID.
   * @param source_uri  the SyncSource URI.
   * @param principalId The principal ID.
   * @throws EntityException
   */
  public void removePopOutboxItem(String GUID,
                                  String source_uri,
                                  long principalId)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start removePopOutboxItem UID " + GUID);
        }
        Connection connection = null;
        PreparedStatement ps = null;
        try {
            lookupDataSource();
            connection = dataSource.getConnection();
            
            // remove from sentpop table
            ps = connection.prepareStatement(DELETE_SENT);
            ps.setString(1, GUID);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            ps.executeUpdate();
            ps.close();
            ps = null;
            
            // remove from cahce
            ps = connection.prepareStatement(DELETE_UNSENT);
            ps.setString(1, GUID);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            ps.executeUpdate();
            ps.close();
            ps = null;
            
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }
        
    }
    
  /**
   *
   *
   * @param f  IMAPFolder the server outbox folder.
   * @param GUID  the Global Unique ID.
   * @param source_uri  the SyncSource URI.
   * @param principalId The principal ID.
   * @throws EntityException
   */
  public void removeImapOutboxItem(IMAPFolder f,
                                   String GUID,
                                   String source_uri,
                                   long principalId)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start removeImapOutboxItem UID " + GUID);
        }
        
        // remove from Mail server Outbox folder
        long FMID = Long.parseLong(Utility.getIdAfterSeparator(GUID));
        try {
            Message msgToDelete = f.getMessageByUID(FMID);
            if (msgToDelete != null) {
                Message[] msgsToDelete = new Message[1];
                msgsToDelete[0] = msgToDelete;
                f.setFlags(msgsToDelete, new Flags(Flags.Flag.DELETED), true);
            }
        } catch (MessagingException e) {
            throw new EntityException(e);
        }
                
        // remove from cache
        Connection connection = null;
        PreparedStatement ps = null;
        try {
            lookupDataSource();
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(DELETE_UNSENT);
            ps.setString(1, GUID);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            ps.executeUpdate();
            ps.close();
            ps = null;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

    }

  /**
    *
    *
    *
    * @param GUID String
    * @param source_uri String
    * @param principalId long
    * @throws EntityException
    */
   public void removeFolder(String GUID,
                            String source_uri,
                            long principalId)
     throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start removeFolder UID " + GUID );
        }
        Connection connection = null;
        PreparedStatement  ps = null ;
        try {
           lookupDataSource();
           connection = dataSource.getConnection();
           ps = connection.prepareStatement(DELETE_FOLDER);
           ps.setString (1, GUID ) ;
           ps.setString (2, source_uri ) ;
           ps.setLong   (3, principalId ) ;
           ps.executeUpdate();
           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

    }  
  
  
  /**
   * This method inserts one of the default folders if needed
   *
   *
   * @param GUID String
   * @param parentId String
   * @param fullPath String
   * @param source_uri String
   * @param principalId long
   * @throws EntityException
   */
  public void insertDefaultFolder(String GUID,
                                  String parentId,
                                  String fullPath,
                                  String source_uri,
                                  long principalId)
    throws EntityException {

      Connection connection = null;
      PreparedStatement  ps = null;

      try {

         lookupDataSource();
         connection = dataSource.getConnection();
         ps = connection.prepareStatement(INSERT_FOLDER);

         ps.setString (1, GUID        ) ;
         ps.setString (2, parentId    ) ;
         ps.setString (3, source_uri  ) ;
         ps.setLong   (4, principalId ) ;
         ps.setString (5, fullPath    ) ;

         ps.executeUpdate();

         ps.close() ;
         ps = null  ;

      } catch (Exception e) {
          throw new EntityException(e);
      } finally {
          DBTools.close(connection, ps, null);
      }

  }


  //-------------------------------------------------------- PROTECTED METHODS


    /**
     * Lookups the datasource
     *
     * JBOSS
     * private String jndiDataSourceName = "java:jdbc/sync4j";
     *
     * TOMCAT
     * private String jndiDataSourceName = "java:comp/env/jdbc/sync4j";
     *
     * @throws Exception if a error occurs during lookup
     */
    protected void lookupDataSource() throws Exception {

        InitialContext ctx = new InitialContext();

        try {
          // try tomcat  i.e. java:comp/env/jdbc/sync4j
          dataSource = (DataSource) ctx.lookup("java:comp/env/"+jndiDataSourceName) ;
        }
        catch (NamingException ne) {
          try {
              // try jboss i.e. java:jdbc/sync4j
              dataSource = (DataSource) ctx.lookup("java:"+jndiDataSourceName) ;
          }
          catch (Exception ex){
            try {
                // try othser j2ee server jdbc/sync4j
                dataSource = (DataSource) ctx.lookup(""+jndiDataSourceName) ;
            }
            catch (Exception e){
              throw new Exception("Data source "+ jndiDataSourceName
                                  + " not found " + e.toString());
            }
          }
        }
      }

  //------------------------------------------------------------ PRIVATE METHODS

    /**
     *
     * @param folder starting folder
     * @param fs all folders container
     * @throws MessagingException
     */
    private void insertSubFolder(IMAPFolder folder, 
                                 IdSpaceGenerator idFolderSpace,
                                 String source_uri, 
                                 long principalId)
       throws MessagingException, EntityException {
        
        javax.mail.Folder[] folderList = folder.list();
        
        IMAPFolder f;

        String fullpath = null;
        String GUID = null;
        String parentGUID = null;
        String parentId = null;
        String parentfullpath = null;
        
        for (int i = 0; i < folderList.length; i++) {
            
            f = ((IMAPFolder) folderList[i]);
            
            fullpath = f.getFullName();
            
            GUID = getGUIDFromFullPath(fullpath, source_uri, principalId);
            
            
            if (GUID == null){
                
                 parentfullpath = Utility.getParentFullpathFromFullpath(fullpath);
                 
                 parentGUID = getGUIDFromFullPath(parentfullpath, source_uri, principalId);
                  
                 parentId = Utility.getIdBeforeSeparator(parentGUID);
                 
                 // insert the missed folder
                 insertFolder(fullpath, parentId,  idFolderSpace, 
                              source_uri, principalId);
                
            }
                        
            insertSubFolder(f, idFolderSpace, source_uri, principalId);
        }
        
    }

}
