/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.items.manager ;


import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Locale;
import java.io.InputStream;
import java.io.ByteArrayInputStream;

import javax.mail.Message;
import javax.mail.Session;
import javax.mail.Folder;
import javax.mail.Flags;
import javax.mail.Transport;
import com.sun.mail.imap.IMAPFolder;
import javax.mail.internet.MimeMessage;
import javax.mail.MessagingException;

import com.funambol.email.pdi.mail.Email;
import com.funambol.email.pdi.mail.EmailItem;
import com.funambol.email.items.dao.EntityDAO;
import com.funambol.email.items.model.ItemMessage;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.CommonMailServerWrapper;
import com.funambol.email.transport.ImapMailServerWrapper;
import com.funambol.email.transport.PopMailServerWrapper;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.exception.*;
import com.funambol.email.items.model.ItemFolder;
import com.funambol.email.items.model.DefaultFolder;
import com.funambol.server.tools.IdSpaceGenerator;
import javax.mail.Address;
import javax.mail.SendFailedException;




/**
 *
 * @version $Id: EntityManager.java,v 1.23 2006/05/15 17:34:57 gbmiglia Exp $
 */
public class EntityManager {



    // -------------------------------------------------------------- Properties

    /**     */
    protected CommonMailServerWrapper cmsw = null;

    /**     */
    protected Logger  log = Logger.getLogger(Def.LOGGER_NAME);

    /**     */
    public EntityDAO ed       = null;

    /**     */
    protected long timeStart = 0;
    protected long timeStop  = 0;

    // ------------------------------------------------------------ Costructors


    /**
     *
     */
    public EntityManager(){
    }

    /**
     *
     *
     * @param msw_e IMailServerWrapper
     * @param jndiDataSourceName String
     * @throws EmailAccessException
     */
    public EntityManager(IMailServerWrapper msw_e, String jndiDataSourceName)
     throws EmailAccessException
    {
       cmsw = (CommonMailServerWrapper)msw_e;
       ed   = new EntityDAO(jndiDataSourceName);
    }

   // ----------------------------------------------------------- Public Methods

   /**
    * deletes an item
    *
    *
    * @param GUID universal id
    * @param source_uri String
    * @param principalId long
    * @throws EntityException
    */
   public void removePopOutboxItem(String GUID,
                                   String source_uri,
                                   long principalId )
    throws EntityException {

       try {
           ed.removePopOutboxItem(GUID, source_uri, principalId);
       }catch (Exception me){
           throw new EntityException(me);
       }

   }

   /**
    * deletes an item
    *
    *
    * @param GUID universal id
    * @param msw IMailServerWrapper
    * @param source_uri String
    * @param principalId long
    * @throws EntityException
    */
   public void removeImapOutboxItem(String GUID,
                                    IMailServerWrapper msw,
                                    String source_uri,
                                    long principalId )
    throws EntityException {

        IMAPFolder f = null;
        
        try {
            ImapMailServerWrapper imsw = (ImapMailServerWrapper)msw;

            f = (IMAPFolder)imsw.getOutboxFolder();

            timeStart = System.currentTimeMillis();

            f.open(Folder.READ_WRITE);
                                 
            ed.removeImapOutboxItem(f, GUID, source_uri, principalId);

        }catch (MessagingException me){
           throw new EntityException(me);
        }catch (EmailAccessException eae){
           throw new EntityException(eae);
        }catch (EntityException e){
          throw new EntityException(e);
        } finally {
           if (f != null){
             try {
               if (f.isOpen()){
                  f.close(true);
                  timeStop = System.currentTimeMillis();
                  if (log.isLoggable(Level.FINEST)) {
                     log.finest("moveSentItem Execution Time: " +
                                (timeStop - timeStart) + " ms");
                  }
               }
             } catch (MessagingException me){
                 throw new EntityException(me);
             }
           }
        }

   }
   

   
   /**
    *
    * move the document from outbox folder to sent folder
    *
    * @param FMID Mail id (in the folder)
    * @param msw IMailServerWrapper
    * @param source_uri String
    * @param principalId long
    * @throws EntityException
    */
   public void moveSentItem(String FMID,
                            IMailServerWrapper msw,
                            String source_uri,
                            long principalId )
      throws EntityException {

        if (msw instanceof ImapMailServerWrapper){
           ImapMailServerWrapper imsw = (ImapMailServerWrapper)msw;
           IMAPFolder sourceFolder = null;
           IMAPFolder targetFolder;
           try {
             sourceFolder = (IMAPFolder)imsw.getOutboxFolder();
             targetFolder = (IMAPFolder)imsw.getSentFolder();
             timeStart = System.currentTimeMillis();
             sourceFolder.open(Folder.READ_WRITE);
             ed.moveImapItem(sourceFolder, targetFolder, Long.parseLong(FMID));
           }catch (MessagingException me){
               throw new EntityException(me);
           }catch (EmailAccessException eae){
               throw new EntityException(eae);
           }catch (EntityException e){
              throw new EntityException(e);
           } finally {
              if (sourceFolder != null){
                try {
                  if (sourceFolder.isOpen()){
                     sourceFolder.close(true);
                     timeStop = System.currentTimeMillis();
                     if (log.isLoggable(Level.FINEST)) {
                        log.finest("moveSentItem Execution Time: " +
                                   (timeStop - timeStart) + " ms");
                     }
                  }
                } catch (MessagingException me){
                    //noinspection ThrowFromFinallyBlock
                    throw new EntityException(me);
                }
              }
           }

        } else if (msw instanceof PopMailServerWrapper){
           try {
               ed.movePopItem(FMID, source_uri, principalId);
           }catch (Exception me){
               throw new EntityException(me);
           }
        }

   }


   /**
    *
    *
    * @param FID String
    * @param emailNew Email
    * @param msw IMailServerWrapper
    * @throws EntityException
    */
    public void sendItem(String FID,
                         Email emailNew,
                         IMailServerWrapper msw)
        throws EntityException    {

        if (log.isLoggable(Level.FINEST)){
          log.finest("EntityManager sendItem start");
        }

        Message msg = null;

        // get Properties
        String read      = "false";
        String forwarded = "false";
        String replied   = "false";
        if (emailNew != null)  {
           read      = (String)emailNew.getRead().getPropertyValue();
           forwarded = (String)emailNew.getForwarded().getPropertyValue();
           replied   = (String)emailNew.getReplied().getPropertyValue();
        }

        if        (msw instanceof ImapMailServerWrapper){
            ImapMailServerWrapper imsw = (ImapMailServerWrapper)msw;
            msg = createMessage(imsw.getSession(), emailNew, FID, true,
                                read, forwarded, replied);
        } else if (msw instanceof PopMailServerWrapper) {
            PopMailServerWrapper pmsw = (PopMailServerWrapper)msw;
            msg = createMessage(pmsw.getSession(), emailNew, FID, true,  
                                read, forwarded, replied);
        }

        // send message
        String subject = null;
        try {
            if (msg != null){
               subject = msg.getSubject();
               timeStart = System.currentTimeMillis();
               
               Transport.send(msg);
               
               timeStop = System.currentTimeMillis();
               if (log.isLoggable(Level.FINEST)) {
                   log.finest("insertItem (send) Execution Time: " +
                              (timeStop - timeStart) + " ms");
               }
            }
        } catch (MessagingException me) {
            
            String errorMessage = "Error sending email " + subject;
            
            String invalidList     = "";
            String validUnsentList = "";
            String validSentList   = "";
            
            Exception ex = me;
            
            do {
                if (ex instanceof SendFailedException) {
                    
                    SendFailedException sfex = (SendFailedException) ex;
                    
                    Address[] invalid = sfex.getInvalidAddresses();
                    if (invalid != null) {
                        for (int i = 0; i < invalid.length; i++) {
                            invalidList = invalidList + " " + invalid[i];
                        }
                        if (!invalidList.trim().equals("")){
                           invalidList = " Invalid address: " + invalidList;
                        }
                    }
                    
                    Address[] validUnsent = sfex.getValidUnsentAddresses();
                    if (validUnsent != null) {
                        for (int i = 0; i < validUnsent.length; i++) {
                            validUnsentList = validUnsentList + " " + validUnsent[i];
                        } 
                        if (!validUnsentList.trim().equals("")){
                           validUnsentList = " Valid unsent address: " + validUnsentList;
                        }
                    }
                    
                    Address[] validSent = sfex.getValidSentAddresses();
                    if (validSent != null) {
                        for (int i = 0; i < validSent.length; i++) {
                            validSentList = validSentList + " " + validSent[i];
                        }
                        if (!validSentList.trim().equals("")){
                           validSentList = " Valid sent address: " + validSentList;
                        }
                    }
                    
                }

                if (ex instanceof MessagingException) {
                    ex = ((MessagingException) ex).getNextException();
                } else {
                    ex = null;
                }
                
            } while (ex != null);
            
            errorMessage = errorMessage + invalidList + validUnsentList + validSentList;
            log.severe(errorMessage);            
            //log.throwing(getClass().getName(), "checkMailCredentials", me);           
            throw new EntityException(errorMessage);
            
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("EntityManager sendItem end");
        }
        
    }

    /**
     *
     * create a javaMail Message from a funambol foundation Email
     *
     * @param s Session
     * @param ctmp Email
     * @param FID String
     * @param toBeSend boolean
     * @return Message
     * @throws EntityException
     */
    public Message createMessage(Session s,
                                 Email ctmp,
                                 String FID,
                                 boolean toBeSend,
                                 String read,
                                 String forwarded,
                                 String replied)
      throws EntityException {

        Message doc;
        try {

            EmailItem emailItem = ctmp.getEmailItem();

            String rfc2822 = emailItem.getStringRFC2822();
            byte[] b = rfc2822.getBytes();
            InputStream is = new ByteArrayInputStream(b);

            doc = new MimeMessage(s, is);

            // if draft set the FLAG
            if (FID.equalsIgnoreCase(Def.FOLDER_DRAFTS_ID)){
               doc.setFlag(Flags.Flag.DRAFT, true);
            }
                        
            // for all folders set eventually the Flags
            boolean flag;
            
            if (read != null){
               flag = (Boolean.valueOf(read)).booleanValue();
               doc.setFlags(new Flags(Flags.Flag.SEEN), flag);
            }
            
            if (forwarded != null){
                //noinspection UnusedAssignment
                flag = (Boolean.valueOf(forwarded)).booleanValue();
               //doc.setFlags(new Flags(Flags.Flag.FORWARDED), flag);
            }
            
            if (replied != null){
               flag = (Boolean.valueOf(replied)).booleanValue();
               doc.setFlags(new Flags(Flags.Flag.ANSWERED), flag);
            }
            
            if (toBeSend){
               // don't set the HEADER_LABEL_SYNCMESSAGE
            } else {
               // we must set the message as "sync message"
               doc.setHeader(Def.HEADER_LABEL_SYNCMESSAGE,
                             Def.HEADER_VALUE_SYNCMESSAGE);
            }

        } catch (Exception e){
            throw new EntityException(e);
        }
        
        return doc;
    }


    /**
     *
     *
     * @param smsg ItemMessage
     * @return Email
     * @throws EntityException
     */
    public Email createFoundationMail(ItemMessage smsg)
     throws EntityException {

        Email mtmp       = new Email();

        String guid      = null;  // parentId + separator + mailId
        String parentId  = null;

        String emailItem = null;
        String encodingType = null;

        String created   = null; // Date created
        String received  = null; // Date received
        String modified  = null; // Date modified
        String read      = null; // boolean read
        String forwarded = null; // boolean forwarded
        String replied   = null; // boolean replied
        String deleted   = null; // boolean deleted
        String flagged   = null; // boolean flagged

        try {

           guid      = smsg.getGUID();
           parentId  = smsg.getParentId();

           emailItem = smsg.getStreamMessage();
           encodingType = smsg.getEncodingType();

           created   = smsg.getCreated();
           received  = smsg.getReceived();
           modified  = smsg.getModified();
           read      = smsg.getRead();
           forwarded = smsg.getForwarded();
           replied   = smsg.getReplied();
           deleted   = smsg.getDeleted();
           flagged   = smsg.getFlagged();

        } catch (Exception e) {
           e.printStackTrace();
        }

        mtmp.getUID().setPropertyValue(guid);
        
        mtmp.getParentId().setPropertyValue(parentId);

        mtmp.getEmailItem().setPropertyValue(emailItem);
        mtmp.getEmailItem().setEncoding(encodingType);

        if (created != null){
          mtmp.getCreated().setPropertyValue(created);
        }
        if (received != null){
          mtmp.getReceived().setPropertyValue(received);
        }
        if (modified != null){
          mtmp.getModified().setPropertyValue(modified);
        }

        if (read != null){
          mtmp.getRead().setPropertyValue(read);
        }
        if (forwarded != null){
          mtmp.getForwarded().setPropertyValue(forwarded);
        }
        if (replied != null){
          mtmp.getReplied().setPropertyValue(replied);
        }
        if (deleted != null){
          mtmp.getDeleted().setPropertyValue(deleted);
        }
        if (flagged != null){
          mtmp.getFlagged().setPropertyValue(flagged);
        }

        return mtmp;
    }

    /**
     *
     * create the foundation folder object 
     *
     * @param f ItemFolder
     * @return Folder
     * @throws EntityException
     */
    public com.funambol.email.pdi.folder.Folder createFoundationFolder(
                                                               ItemFolder f)
      throws EntityException {

        com.funambol.email.pdi.folder.Folder fout = new 
                                         com.funambol.email.pdi.folder.Folder();

        fout.getUID().setPropertyValue(f.getGUID());

        fout.getParentId().setPropertyValue(f.getParentId());

        fout.getName().setPropertyValue(f.getName());

        fout.getCreated().setPropertyValue(f.getCreated());

        fout.getAccessed().setPropertyValue(f.getAccessed());

        fout.getModified().setPropertyValue(f.getModified());

        fout.getRole().setPropertyValue(f.getRole());

        //fout.getAttributes().setPropertyValue(f.getAttributes);

        return fout;
    }

    /**
     *
     * insert the default folder if needed
     *
     *
     * @param df DefaultFolder
     * @param source_uri String
     * @param principalId long
     * @throws EntityException
     */
    public void  insertDefaultFolder(DefaultFolder df,
                                     String source_uri,
                                     long principalId)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertDefaultFolder start");
        }

        String GUID;
        String parentId;
        String fullpath;
        String defaultFolder;

        try {

          GUID          = Utility.getGUID(Def.FOLDER_ROOT_ID, df.getInboxId());
          parentId      = Def.FOLDER_ROOT_ID ;
          fullpath      = df.getInboxName();
          defaultFolder = this.ed.getFullPathFromGUID(GUID,
                                                      source_uri,
                                                      principalId);
          if (defaultFolder == null){
              this.ed.insertDefaultFolder(GUID, parentId, fullpath,
                                          source_uri, principalId);
          }
          GUID          = Utility.getGUID(Def.FOLDER_ROOT_ID, df.getOutboxId());
          parentId      = Def.FOLDER_ROOT_ID ;
          fullpath      = df.getOutboxName();
          defaultFolder = this.ed.getFullPathFromGUID(GUID,
                                                      source_uri,
                                                      principalId);
          if (defaultFolder == null){
              this.ed.insertDefaultFolder(GUID,
                                          parentId,
                                          fullpath,
                                          source_uri,
                                          principalId);
          }
          GUID          = Utility.getGUID(Def.FOLDER_ROOT_ID, df.getSentId());
          parentId      = Def.FOLDER_ROOT_ID ;
          fullpath      = df.getSentName();
          defaultFolder = this.ed.getFullPathFromGUID(GUID,
                                                      source_uri,
                                                      principalId);
          if (defaultFolder == null){
              this.ed.insertDefaultFolder(GUID,
                                          parentId,
                                          fullpath,
                                          source_uri,
                                          principalId);
          }
          GUID          = Utility.getGUID(Def.FOLDER_ROOT_ID, df.getDraftId());
          parentId      = Def.FOLDER_ROOT_ID ;
          fullpath      = df.getDraftName();
          defaultFolder = this.ed.getFullPathFromGUID(GUID,
                                                      source_uri,
                                                      principalId);
          if (defaultFolder == null){
              this.ed.insertDefaultFolder(GUID,
                                          parentId,
                                          fullpath,
                                          source_uri,
                                          principalId);
          }
          GUID          = Utility.getGUID(Def.FOLDER_ROOT_ID, df.getTrashId());
          parentId      = Def.FOLDER_ROOT_ID ;
          fullpath      = df.getTrashName();
          defaultFolder = this.ed.getFullPathFromGUID(GUID,
                                                      source_uri,
                                                      principalId);
          if (defaultFolder == null){
              this.ed.insertDefaultFolder(GUID,
                                          parentId,
                                          fullpath,
                                          source_uri,
                                          principalId);
          }

       } catch (EntityException ee){
           throw new EntityException("Error inserting the default folder", ee);
       }

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertDefaultFolder end" );
        }

    }
    
   /**
     *
     * Check if the local DB folders are consistent with the MailBox folders
     *
     *
     * @param source_uri String
     * @param principalId long
     * @throws EntityException
     */
    public void  insertCustomFolder(IMailServerWrapper msw,
                                    boolean allMailboxActivation,
                                    IdSpaceGenerator idFolderSpace,
                                    String source_uri, 
                                    long principalId)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertCustomFolder start");
        }

        try {
            
            if (msw instanceof ImapMailServerWrapper){
                           
               ImapMailServerWrapper imsw = (ImapMailServerWrapper)msw;

                if (allMailboxActivation){
                    String     defFullpath;
                    IMAPFolder imapf;

                    // insert all folders in Inboxbox + inboxbox and so on
                    defFullpath = imsw.getDefaultFolder().getInboxName();
                    imapf       = imsw.getInboxFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                    defFullpath = imsw.getDefaultFolder().getOutboxName();
                    imapf       = imsw.getOutboxFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                    defFullpath = imsw.getDefaultFolder().getSentName();
                    imapf       = imsw.getSentFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                    defFullpath = imsw.getDefaultFolder().getDraftName();
                    imapf       = imsw.getDraftsFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                    defFullpath = imsw.getDefaultFolder().getTrashName();
                    imapf       = imsw.getTrashFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                }
                              
            }
               
        } catch (EmailAccessException ee){
           throw new EntityException("Error getting all folder", ee);
        } catch (EntityException ee){
           throw new EntityException("Error inserting the custom folder", ee);
        }

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertCustomFolder end" );
        }

    }
    
    
    /**
     *
     * Check if the item is a folder or an email
     * using the SyncItemKey
     *
     *
     * @param parentId String
     * @param objectId String
     * @param source_uri String
     * @param principalId long
     * @return boolean
     * @throws EntityException
     */
    public boolean isEmail(String parentId,
                           String objectId,
                           String source_uri,
                           long principalId)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager isEmail start");
        }

        boolean ismail = false;
        ItemFolder it;


        try {

           it = this.ed.getFolderFromUID(Utility.getGUID(parentId, objectId), 
                                         source_uri, 
                                         principalId);

           if (it == null){
              
              ismail = true;
               
              /* 
              // try to get a mail
              String parentGUID = it.getParentId();
              String parentfullpath = this.ed.getFullPathFromGUID(parentGUID,
                                                                  source_uri,
                                                                  principalId);              
              Folder f = 
                     this.cmsw.getMailDefaultFolder().getFolder(parentfullpath);
              
              try {
                  long uid = Long.parseLong(objectId);
                  // if the parse can convert the id we have to go
                  // on a IMAPFolder
                  IMAPFolder imapFolder = (IMAPFolder)f;
                  Message m = imapFolder.getMessageByUID(uid);
                  if (m != null){
                     ismail = true;
                  }
              } catch (NumberFormatException  nfe){
                  // if the parse can't convert the id we have to go
                  // on a POP3Folder
                  POP3Folder pop3Folder = (POP3Folder)f;
                  Message m = this.ed.getMessageByUID(pop3Folder, objectId);
                  if (m != null){
                     ismail = true;
                  }
              }
              */
           }

        } catch (EntityException ee){
           throw new EntityException("Error determinig object type (mail/folder)",
                                     ee);
        }

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager isEmail end (isEmail :" + ismail + ")" );
        }

        return ismail;

    }

    // -------------------------------------------------------- Protected Methods

     /**
      * convert Message into ServerMessages
      *
      *
      * @param GUID unique folder id
      * @param name folder name
      * @param FID parent id
      * @return ItemFolder
      * @throws EntityException
      */
     public ItemFolder setItemFolder(String GUID,
                                        String name,
                                        String FID,
                                        String folderRole,
                                        String creationDate)
       throws EntityException {

         if (log.isLoggable(Level.FINEST)) {
             log.finest("setItemFolder() start " + GUID);
         }

         ItemFolder    clientFoler;

         try {



             clientFoler = new ItemFolder(GUID,
                                         name,
                                         FID,
                                         creationDate,
                                         null, // accessed
                                         null, // modified
                                         folderRole,
                                         null // attributes
                                         );

         } catch (Exception e){
             throw new EntityException(e);
         }

         if (log.isLoggable(Level.FINEST)) {
             log.finest("setItemFolder() - end ");
         }

         return clientFoler;

     }

     /**
      *
      * convert Message into ServerMessages
      *
      * @param GUID unique mail id
      * @param msg Message
      * @param parentId String
      * @param loc Locale
      * @param encodingType String
      * @return ItemMessage
      * @throws EntityException
      */
     public ItemMessage setItemMessage(String GUID,
                                       Message msg,
                                       String parentId,
                                       Locale loc,
                                       String encodingType)
       throws EntityException {

         ItemMessage   clientMsg;
         String        message_id;
         String        stream;
         String        created;
         String        received;
         String        modified;
         String        read;
         String        forwarded;
         String        replied;
         String        deleted;
         String        flagged;

         try {

             // header message-id
             message_id = Utility.getHeaderMessageID(msg);

             // message
             stream = Utility.getStreamFromMessage(msg, encodingType);

             //origination date
             created = Utility.UtcFromDate(Utility.getHeaderDate(msg, loc));
             //received date
             received = Utility.getReceivedDate(msg, loc);
             //modified date
             modified = null;

             // read
             read = String.valueOf(msg.isSet(Flags.Flag.SEEN));
             // forwarded
             forwarded = null;
             // replied
             replied = null;
             // deleted
             deleted = String.valueOf(msg.isSet(Flags.Flag.DELETED));
             // flagged
             flagged = String.valueOf(msg.isSet(Flags.Flag.FLAGGED));

             clientMsg = new ItemMessage(GUID,
                                         parentId,
                                         message_id,
                                         msg,
                                         stream,
                                         encodingType,
                                         created,
                                         received,
                                         modified,
                                         read,
                                         forwarded,
                                         replied,
                                         deleted,
                                         flagged);

         } catch (MessagingException e){
             throw new EntityException(e);
         }

         return clientMsg;

     }

}
