/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.items.manager ;


import com.funambol.email.engine.cache.CrcSyncItemInfo;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.EntityException;
import com.funambol.email.items.dao.PopEntityDAO;
import com.funambol.email.items.model.EmailFilter;
import com.funambol.email.items.model.ItemFolder;
import com.funambol.email.items.model.ItemMessage;
import com.funambol.email.pdi.mail.Email;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.PopMailServerWrapper;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.server.tools.IdSpaceGenerator;
import com.sun.mail.pop3.POP3Folder;

import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.logging.Level;


/**
 * @version $Id: PopEntityManager.java,v 1.22 2006/05/15 17:34:57 gbmiglia Exp $
 */
public class PopEntityManager extends EntityManagerFilter {

    // -------------------------------------------------------------- Properties

    /**     */
    public PopMailServerWrapper pmsw = null;

    /**     */
    public PopEntityDAO ped = null;

    // ------------------------------------------------------------ Costructors


    /**
     *
     */
    public PopEntityManager() {
    }

    /**
     * @param msw_e              IMailServerWrapper
     * @param jndiDataSourceName String
     * @throws EmailAccessException
     */
    public PopEntityManager(IMailServerWrapper msw_e, String jndiDataSourceName)
            throws EmailAccessException {
        pmsw = (PopMailServerWrapper) msw_e;
        ped = new PopEntityDAO(jndiDataSourceName);
    }

    // ---------------------------------------------------------- Public Methods


    /**
     * @param FID         parent id
     * @param emailNew    Email
     * @param idSentSpace IdSpaceGenerator
     * @param source_uri  String
     * @param principalId long
     * @return Email
     * @throws EntityException
     */
    public Email addEmail(String FID,
                          Email emailNew,
                          IdSpaceGenerator idSentSpace,
                          String source_uri,
                          long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("PopEmail insertEmail start");
        }

        Email emailOut;

        // get Properties
        String read      = "false";
        String forwarded = "false";
        String replied   = "false";
        if (emailNew != null) {
            read      = (String) emailNew.getRead().getPropertyValue();
            forwarded = (String) emailNew.getForwarded().getPropertyValue();
            replied   = (String) emailNew.getReplied().getPropertyValue();
        }

        if (emailNew != null && emailNew.getEmailItem() != null &&
                Def.ENCODE_QUOTED_PRINTABLE.equals(emailNew.getEmailItem().getEncoding())) {
            try {
                emailNew.getEmailItem().setPropertyValue(
                        Utility.decode(emailNew.getEmailItem().getStringRFC2822(),
                                Def.ENCODE_QUOTED_PRINTABLE));
                emailNew.getEmailItem().setEncoding(null);
            }
            catch (MessagingException e) {
                throw new EntityException(e);
            }
        }

        try {
            Message msg = createMessage(this.pmsw.getSession(), emailNew,
                                        FID, false, read, forwarded, replied);
            
            String GUID = ped.addEmail(FID, msg,  idSentSpace, 
                                       source_uri, principalId);
            
            ItemMessage im = setItemMessage(GUID, msg, FID, 
                                            this.pmsw.getLocale(), null);
            
            // convert Message in Email
            emailOut = createFoundationMail(im);
            
        } catch (EntityException e) {
            throw new EntityException(e);
        }


        if (log.isLoggable(Level.FINEST)) {
            log.finest("PopEmail insertEmail end");
        }

        return emailOut;

    }

    /**
     * clean all folder and email except the 5 main folders
     *
     * @param source_uri  String
     * @param principalId long
     * @throws EntityException
     */
    public void clearAllItems(String source_uri, long principalId)
            throws EntityException {

        POP3Folder f = null;
        if (log.isLoggable(Level.FINEST)) {
            log.finest("Pop3Email Delete all emails in Inbox Folder");
        }

        try {

            // we can remove only the inbox folder email
            f = this.pmsw.getInboxFolder();
            timeStart = System.currentTimeMillis();
            f.open(Folder.READ_WRITE);
            this.ped.removeAllEmail(f);

            // the local sent messages is already deleted.

            // we can't remove all folder

        } catch (MessagingException me) {
            throw new EntityException(me);
        } catch (EmailAccessException e) {
            throw new EntityException(e);
        } finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("insertItem Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                } catch (MessagingException me) {
                    //noinspection ThrowFromFinallyBlock
                    throw new EntityException(me);
                }
            }
        }

    }


    /**
     * deletes an email
     *
     * @param parentId    folder id
     * @param mailId      mail id in the folder.
     * @param source_uri  String
     * @param principalId long
     * @throws EntityException
     */
    public void removeEmail(String parentId,
                            String mailId,
                            String source_uri,
                            long principalId)
            throws EntityException {


        if (parentId.equalsIgnoreCase(Def.FOLDER_INBOX_ID)) {

            POP3Folder f = null;

            try {

                f = (POP3Folder)
                        this.pmsw.getMailDefaultFolder().getFolder(Def.FOLDER_INBOX_ENG);

                timeStart = System.currentTimeMillis();
                f.open(Folder.READ_WRITE);
                ped.removeEmail(f, mailId);

            } catch (MessagingException me) {
                throw new EntityException(me);
            } catch (EntityException e) {
                throw new EntityException(e);
            } finally {
                if (f != null) {
                    try {
                        if (f.isOpen()) {
                            f.close(true);
                            timeStop = System.currentTimeMillis();
                            if (log.isLoggable(Level.FINEST)) {
                                log.finest("removeItem Execution Time: " +
                                        (timeStop - timeStart) + " ms");
                            }
                        }
                    } catch (MessagingException me) {
                        //noinspection ThrowFromFinallyBlock
                        throw new EntityException(me);
                    }
                }
            }
        }


        if (parentId.equalsIgnoreCase(Def.FOLDER_SENT_ID)
                ||
                parentId.equalsIgnoreCase(Def.FOLDER_OUTBOX_ID)) {

            try {
                ped.removeEmail(Utility.getGUID(parentId, mailId),
                        source_uri, principalId);

            } catch (Exception me) {
                throw new EntityException(me);
            }
        }

    }

    /**
     * @param filter      EmailFilter
     * @param source_uri  The SyncSource URI.
     * @param principalId The principal ID.
     * @return The all items fulfilling the filter requirements as an array of Strings.
     * @throws EntityException
     */
    public String[] getAllEmails(EmailFilter filter,
                                 String source_uri,
                                 long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Start - getAllItems() ");
        }

        String[] allitems;
        String[] inboxList;
        String[] sentList;

        try {

            POP3Folder inbox = (POP3Folder)
                    this.pmsw.getMailDefaultFolder().getFolder(Def.FOLDER_INBOX_ENG);

            inboxList = getAllEmailsInbox(inbox, filter);

            sentList = getAllEmailsSent(filter, source_uri, principalId);

        } catch (Exception e) {
            throw new EntityException("Error Getting all emails (" +
                    e.getMessage() + ")");
        }

        allitems = Utility.mergeArray(inboxList, null, null, sentList, null);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("End - getAllItems() ");
        }

        return allitems;
    }


    /**
     * @param filter      EmailFilter
     * @param source_uri  String
     * @param principalId long
     * @return CrcSyncItemInfo[]
     * @throws EntityException
     */
    public CrcSyncItemInfo[] getAllEmailInfo(EmailFilter filter,
                                             String source_uri,
                                             long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo Start");
        }

        CrcSyncItemInfo[] syncItemInfos;
        CrcSyncItemInfo[] iInfos;
        CrcSyncItemInfo[] sInfos;

        try {

            POP3Folder inbox = (POP3Folder)
                    this.pmsw.getMailDefaultFolder().getFolder(Def.FOLDER_INBOX_ENG);

            iInfos = this.getEmailsInboxInfo(inbox, filter);

            sInfos = this.getEmailsSentInfo(filter, source_uri, principalId);
        } catch (Exception e) {
            throw new EntityException("Error Getting all emails (" +
                    e.getMessage() + ")");
        }

        syncItemInfos = Utility.mergeArray(iInfos, null, null, sInfos, null);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo List: " + syncItemInfos.length + " Document");
        }

        return syncItemInfos;

    }

    /**
     * the pop-like connector gets only the default folders
     * (inbox, outbox, sent)
     *
     * @return The info of the syncitems as an array.
     * @throws EntityException
     */
    public CrcSyncItemInfo[] getAllFolderInfo()
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("POPManager getAllFolderInfo Start");
        }

        CrcSyncItemInfo[] syncItemInfos = new CrcSyncItemInfo[3];

        CrcSyncItemInfo inbox = new CrcSyncItemInfo();
        inbox.setKey(Def.FOLDER_INBOX_GUID);
        inbox.setLastCrc(1);
        syncItemInfos[0] = inbox;

        CrcSyncItemInfo outbox = new CrcSyncItemInfo();
        outbox.setKey(Def.FOLDER_OUTBOX_GUID);
        outbox.setLastCrc(1);
        syncItemInfos[1] = outbox;

        CrcSyncItemInfo sent = new CrcSyncItemInfo();
        sent.setKey(Def.FOLDER_SENT_GUID);
        sent.setLastCrc(1);
        syncItemInfos[2] = sent;

        return syncItemInfos;

    }

    /**
     * @param filter      EmailFilter
     * @param UID         mail id (folderid / mailid)
     * @param source_uri  String
     * @param principalId long
     * @return Email
     * @throws EntityException
     */
    public Email getEmailFromUID(EmailFilter filter,
                                 String UID,
                                 String source_uri,
                                 long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemByUID start " + UID);
        }

        String parentId = Utility.getIdBeforeSeparator(UID);
        String mailId   = Utility.getIdAfterSeparator(UID);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemByUID folder: " + parentId + " mail " + mailId);
        }

        ItemMessage clientMsg = null;

        if (parentId.equalsIgnoreCase(Def.FOLDER_INBOX_ID)) {
            clientMsg = this.getEmailInbox(parentId, mailId, filter,
                                           source_uri, principalId);
        }

        if (parentId.equalsIgnoreCase(Def.FOLDER_SENT_ID) ||
            parentId.equalsIgnoreCase(Def.FOLDER_OUTBOX_ID)) {

            clientMsg = this.getEmailSent(parentId, mailId, filter,
                                          source_uri, principalId);
        }

        Email emailOut = null;
        if (clientMsg != null) {
            try {
                emailOut = createFoundationMail(clientMsg);
            } catch (Exception e) {
                throw new EntityException(e);
            }
        }

        return emailOut;

    }

    /**
     * @param GUID        folder id
     * @param source_uri  String
     * @param principalId long
     * @return Folder
     * @throws EntityException
     */
    public com.funambol.email.pdi.folder.Folder getFolderFromUID(String GUID,
                                                                 String source_uri,
                                                                 long principalId)
            throws EntityException {


        if (log.isLoggable(Level.FINEST)) {
            log.finest("getFolderByUID start " + GUID);
        }

        com.funambol.email.pdi.folder.Folder folderOut;
        ItemFolder itemFolder = null;
        String creationDate;


        try {

            creationDate = Utility.getFolderCreationDate();

            if (GUID.equalsIgnoreCase(Def.FOLDER_INBOX_GUID)) {
                itemFolder = new ItemFolder(GUID,
                        this.pmsw.getDefaultFolder().getInboxName(),
                        Def.FOLDER_ROOT_ID,
                        creationDate,
                        null,
                        null,
                        Def.FOLDER_INBOX_ROLE,
                        null);
            } else if (GUID.equalsIgnoreCase(Def.FOLDER_OUTBOX_GUID)) {
                itemFolder = new ItemFolder(GUID,
                        this.pmsw.getDefaultFolder().getOutboxName(),
                        Def.FOLDER_ROOT_ID,
                        creationDate,
                        null,
                        null,
                        Def.FOLDER_OUTBOX_ROLE,
                        null);

            } else if (GUID.equalsIgnoreCase(Def.FOLDER_SENT_GUID)) {
                itemFolder = new ItemFolder(GUID,
                        this.pmsw.getDefaultFolder().getSentName(),
                        Def.FOLDER_ROOT_ID,
                        creationDate,
                        null,
                        null,
                        Def.FOLDER_SENT_ROLE,
                        null);
            }

            folderOut = createFoundationFolder(itemFolder);

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return folderOut;

    }

    /**
     * @param name        folder name
     * @param parentId    parent folder id
     * @param source_uri  String
     * @param principalId long
     * @return Folder
     * @throws EntityException
     */
    public com.funambol.email.pdi.folder.Folder getFolderFromName(String name,
                                                                  String parentId,
                                                                  String source_uri,
                                                                  long principalId)
            throws EntityException {


        if (log.isLoggable(Level.FINEST)) {
            log.finest("getFolderByName start " + name + " FID " + parentId);
        }

        com.funambol.email.pdi.folder.Folder folderOut = null;
        ItemFolder itemFolder = null;
        String creationDate;

        /**
         * we should add some control.
         * We can use the device information
         */
        String nametmp = Utility.convertNokiaToDefault(name,
                this.pmsw.getDefaultFolder());
        // it is a nokia folder
        if (nametmp != null) {
            name = nametmp;
        }

        try {

            creationDate = Utility.getFolderCreationDate();

            if (name.equalsIgnoreCase(this.pmsw.getDefaultFolder().getInboxName())) {
                itemFolder = new ItemFolder(Def.FOLDER_INBOX_GUID,
                        this.pmsw.getDefaultFolder().getInboxName(),
                        Def.FOLDER_ROOT_ID,
                        creationDate,
                        null,
                        null,
                        Def.FOLDER_INBOX_ROLE,
                        null);
            } else if (name.equalsIgnoreCase(this.pmsw.getDefaultFolder().getOutboxName())) {
                itemFolder = new ItemFolder(Def.FOLDER_OUTBOX_GUID,
                        this.pmsw.getDefaultFolder().getOutboxName(),
                        Def.FOLDER_ROOT_ID,
                        creationDate,
                        null,
                        null,
                        Def.FOLDER_OUTBOX_ROLE,
                        null);

            } else if (name.equalsIgnoreCase(this.pmsw.getDefaultFolder().getSentName())) {
                itemFolder = new ItemFolder(Def.FOLDER_SENT_GUID,
                        this.pmsw.getDefaultFolder().getSentName(),
                        Def.FOLDER_ROOT_ID,
                        creationDate,
                        null,
                        null,
                        Def.FOLDER_SENT_ROLE,
                        null);
            }

            if (itemFolder != null) {
                try {
                    folderOut = createFoundationFolder(itemFolder);
                } catch (Exception e) {
                    throw new EntityException(e);
                }
            }

        } catch (Exception e) {
            throw new EntityException(e);
        }


        return folderOut;

    }

    /**
     * checks if a contact is already inserted in the db
     * and returns the Document in the DB
     * we use firstname, lastname as search criteria
     *
     * @param parentId    parent folder id
     * @param mailId      mail id (in the folder not unique)
     * @param ctmp        Email
     * @param source_uri  String
     * @param principalId long
     * @return Email
     * @throws EntityException
     */
    public String getEmailFromClause(String parentId,
                                     String mailId,
                                     Email ctmp,
                                     String source_uri,
                                     long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getItemByWhereClause start ");
        }

        String GUID = null;

        String messageIdExt;
        Message msgExt;

        // get Properties
        String read      = "false";
        String forwarded = "false";
        String replied   = "false";
        if (ctmp != null) {
            read      = (String) ctmp.getRead().getPropertyValue();
            forwarded = (String) ctmp.getForwarded().getPropertyValue();
            replied   = (String) ctmp.getReplied().getPropertyValue();
        }

        try {
            msgExt = createMessage(this.pmsw.getSession(), ctmp,  parentId,
                                   false, read, forwarded, replied);
            messageIdExt = Utility.getHeaderMessageID(msgExt);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        if (parentId.equalsIgnoreCase(Def.FOLDER_INBOX_ID)) {

            POP3Folder f = null;
            try {

                f = (POP3Folder) this.pmsw.getMailDefaultFolder().getFolder(Def.FOLDER_INBOX_ENG);

                timeStart = System.currentTimeMillis();
                f.open(Folder.READ_WRITE);

                if (messageIdExt != null) {
                    GUID = ped.getEmailFromMessageID(f, parentId, messageIdExt).getGUID();
                }

            } catch (Exception e) {
                throw new EntityException(e);
            } finally {
                if (f != null) {
                    try {
                        if (f.isOpen()) {
                            f.close(true);
                            timeStop = System.currentTimeMillis();
                            if (log.isLoggable(Level.FINEST)) {
                                log.finest("getItemByWhereClause Execution Time: " +
                                        (timeStop - timeStart) + " ms");
                            }
                        }
                    } catch (MessagingException me) {
                        //noinspection ThrowFromFinallyBlock
                        throw new EntityException(me);
                    }
                }
            }
        }

        if (parentId.equalsIgnoreCase(Def.FOLDER_SENT_ID) ||
            parentId.equalsIgnoreCase(Def.FOLDER_OUTBOX_ID)) {

            try {
                if (messageIdExt != null) {
                    GUID = ped.getEmailFromMessageID(messageIdExt,
                                                     this.pmsw.getSession(),
                                                     source_uri,
                                                     principalId).getGUID();
                }
            } catch (Exception e) {
                throw new EntityException(e);
            }
        }

        return GUID;
    }


    /**
     * @param FID         parent folder id
     * @param FMID        mail id (in the folder, not unique)
     * @param filter      EmailFilter
     * @param source_uri  String
     * @param principalId long
     * @return ItemMessage
     * @throws EntityException
     */
    private ItemMessage getEmailInbox(String FID,
                                      String FMID,
                                      EmailFilter filter,
                                      String source_uri,
                                      long principalId)
            throws EntityException {

        ItemMessage im = null;
        POP3Folder f = null;

        try {

            f = (POP3Folder)
                    this.pmsw.getMailDefaultFolder().getFolder(Def.FOLDER_INBOX_ENG);

            timeStart = System.currentTimeMillis();
            
            f.open(Folder.READ_WRITE);
            
            Message msg = ped.getEmailFromUID(f, FMID);

            im = this.getPart(f, FMID, filter, msg, source_uri, principalId);

        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("getItemByUID Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                }
                catch (MessagingException me) {
                    //noinspection ThrowFromFinallyBlock
                    throw new EntityException(me);
                }
            }
        }

        return im;
    }

    /**
     * @param FID         String
     * @param FMID        String
     * @param filter      EmailFilter
     * @param source_uri  String
     * @param principalId long
     * @return ItemMessage
     * @throws EntityException
     */
    private ItemMessage getEmailSent(String FID,
                                     String FMID,
                                     EmailFilter filter,
                                     String source_uri,
                                     long principalId)
            throws EntityException {

        ItemMessage clientMsg;
        String encodingType;
        boolean asciiOnly;

        try {
            String GUID = Utility.getGUID(FID, FMID);
            Message msg = ped.getEmailFromUID(GUID, this.pmsw.getSession(),
                    source_uri, principalId);
            clientMsg = setItemMessage(GUID, msg, FID, this.pmsw.getLocale(), null);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        return clientMsg;
    }

    /**
     * get messages from INBOX folder
     *
     * @param f      POP3Folder
     * @param filter EmailFilter
     * @return String[]
     * @throws EntityException
     */
    private String[] getAllEmailsInbox(POP3Folder f, EmailFilter filter)
            throws EntityException {

        String[] msgIds = null;

        try {

            timeStart = System.currentTimeMillis();
            f.open(Folder.READ_WRITE);

            Message[] messages = this.ped.getAllEmailsInbox(f, filter, this.pmsw.getLocale());
            if (messages != null) {
                int numMsgs = messages.length;
                msgIds = new String[numMsgs];
                String FMID;

                for (int i = 0; i < numMsgs; i++) {
                    FMID = String.valueOf(f.getUID(messages[i]));
                    msgIds[i] = Utility.getGUID(Def.FOLDER_INBOX_ID, FMID);
                }
            }

        } catch (MessagingException e) {
            throw new EntityException(e);
        } finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("getItems Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                } catch (MessagingException me) {
                    //noinspection ThrowFromFinallyBlock
                    throw new EntityException(me);
                }
            }
        }

        return msgIds;

    }


    /**
     * get messages from folders
     *
     * @param filter
     * @param source_uri  String
     * @param principalId long
     * @return String[]
     * @throws EntityException
     */
    private String[] getAllEmailsSent(EmailFilter filter,
                                      String source_uri,
                                      long principalId)
            throws EntityException {

        String[] msgIds;

        try {

            msgIds = this.ped.getAllEmailsSent(filter, source_uri, principalId);

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return msgIds;

    }

    /**
     * the pop-like connector gets only the default folders
     * (inbox, outbox, sent)
     *
     * @return the array with the folder ids String[]
     * @throws EntityException
     */
    public String[] getAllFolders() throws EntityException {

        return new String[]{
                Def.FOLDER_INBOX_GUID,
                Def.FOLDER_OUTBOX_GUID,
                Def.FOLDER_SENT_GUID
        };

    }

    /**
     * get messages from folders
     *
     * @param f      POP3Folder
     * @param filter EmailFilter
     * @return CrcSyncItemInfo[]
     * @throws EntityException
     */
    private CrcSyncItemInfo[] getEmailsInboxInfo(POP3Folder f, EmailFilter filter)
            throws EntityException {

        List allSyncItemInfo = new ArrayList();
        CrcSyncItemInfo syncItemInfo;

        try {

            timeStart = System.currentTimeMillis();
            f.open(Folder.READ_WRITE);

            // get message
            Message[] items = ped.getAllEmailsInbox(f, filter, this.pmsw.getLocale());

            Message msg;
            String GUID;
            long lastCrc;
            int itemsNum = 0;

            if (items != null) {
                itemsNum = items.length;
            }

            for (int i = 0; i < itemsNum; i++) {

                msg = items[i];

                lastCrc = Utility.setLastCrc(msg);
                GUID = Utility.getGUID(Def.FOLDER_INBOX_ID, f.getUID(msg));

                syncItemInfo = new CrcSyncItemInfo();
                syncItemInfo.setLastCrc(lastCrc);
                syncItemInfo.setKey(GUID);

                allSyncItemInfo.add(syncItemInfo);

            }

        } catch (MessagingException e) {
            throw new EntityException(e);
        }
        finally {
            if (f != null) {
                try {
                    if (f.isOpen()) {
                        f.close(true);
                        timeStop = System.currentTimeMillis();
                        if (log.isLoggable(Level.FINEST)) {
                            log.finest("getItemsInfo Execution Time: " +
                                    (timeStop - timeStart) + " ms");
                        }
                    }
                }
                catch (MessagingException me) {
                    //noinspection ThrowFromFinallyBlock
                    throw new EntityException(me);
                }
            }
        }

        return (CrcSyncItemInfo[]) allSyncItemInfo.toArray(new CrcSyncItemInfo[0]);

    }


    /**
     * get messages from folders
     *
     * @param filter
     * @param source_uri  String
     * @param principalId long
     * @return CrcSyncItemInfo[]
     * @throws EntityException
     */
    private CrcSyncItemInfo[] getEmailsSentInfo(EmailFilter filter,
                                                String source_uri,
                                                long principalId)
            throws EntityException {

        List allSyncItemInfo = new ArrayList();
        CrcSyncItemInfo syncItemInfo;

        try {

            // get message
            String[] ids = ped.getAllEmailsSent(filter,
                    source_uri,
                    principalId);

            String id;
            String key;
            long lastCrc;
            int itemsNum = 0;

            if (ids != null) {
                itemsNum = ids.length;
            }

            for (int i = 0; i < itemsNum; i++) {

                id = ids[i];

                key = id;
                lastCrc = 1;

                if (log.isLoggable(Level.FINEST)) {
                    log.finest("Pop3Email getItemsInfo info: " +
                            " uid " + key + " crc " + lastCrc);
                }
                syncItemInfo = new CrcSyncItemInfo();
                syncItemInfo.setLastCrc(lastCrc);
                syncItemInfo.setKey(key);
                allSyncItemInfo.add(syncItemInfo);

            }

        } catch (Exception e) {
            throw new EntityException(e);
        }

        return (CrcSyncItemInfo[]) allSyncItemInfo.toArray(new CrcSyncItemInfo[0]);

    }

    /**
     * @param f           POP3Folder
     * @param FMID        mail id in the folder , not unique
     * @param filter      EmailFilter
     * @param msg         Message
     * @param source_uri  String
     * @param principalId long
     * @return ItemMessage
     * @throws EntityException
     */
    private ItemMessage getPart(POP3Folder f,
                                String FMID,
                                EmailFilter filter,
                                Message msg,
                                String source_uri,
                                long principalId)
            throws EntityException {

        ItemMessage im = null;
        if (msg != null) {

            try {

                String parentfullpath = f.getFullName();
                if (parentfullpath.equals("INBOX")) {
                    parentfullpath = Def.FOLDER_INBOX_ENG;
                }

                String FID = this.ped.getGUIDFromFullPath(parentfullpath,
                                                          source_uri,
                                                          principalId);
                Locale loc = this.pmsw.getLocale();

                Session session = this.pmsw.getSession();

                if (filter.getSize() == Def.FILTER_SIZE_H) {
                    im = getPart_H(session, FID, FMID, msg, loc);
                } else if (filter.getSize() == Def.FILTER_SIZE_H_B) {
                    im = getPart_HB(session, FID, FMID, msg, loc, -1);
                } else if (filter.getSize() == Def.FILTER_SIZE_H_B_PERC) {
                    im = getPart_HB(session, FID, FMID, msg, loc, filter.getNumBytes());
                } else if (filter.getSize() == Def.FILTER_SIZE_H_B_A) {
                    im = getPart_HBA(session, FID, FMID, msg, loc, -1);
                } else if (filter.getSize() == Def.FILTER_SIZE_H_B_A_PERC) {
                    im = getPart_HBA(session, FID, FMID, msg, loc, filter.getNumBytes());
                }
            } catch (EntityException me) {
                throw new EntityException(me);
            }

        }
        return im;
    }


}
