/*
 * <FUNAMBOLCOPYRIGHT>
 * Copyright (C) 2006 FUNAMBOL.
 * All Rights Reserved.  No use, copying or distribution of this
 * work may be made except in accordance with a valid license
 * agreement from FUNAMBOL.  This notice must be
 * included on all copies, modifications and derivatives of this
 * work.
 *
 * FUNAMBOL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
 * OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. FUNAMBOL SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * </FUNAMBOLCOPYRIGHT>
 */
package com.funambol.email.pdi.converter;

import java.util.Vector;
import java.util.HashMap;
import java.util.TimeZone;
import java.util.Iterator;

import com.funambol.foundation.common.StringTools;
import com.funambol.foundation.pdi.converter.BaseConverter;
import com.funambol.foundation.pdi.converter.ConverterException;
import com.funambol.foundation.pdi.common.Property;

import com.funambol.email.pdi.folder.*;


/**
 * This object is a converter from a Folder object model to an XML string.
 *
 * @version $Id: FolderToXML.java,v 1.2 2006/03/28 16:13:17 gbmiglia Exp $
 */
public class FolderToXML extends BaseConverter implements IFOLDER
{

    // ------------------------------------------------------------ Private Data
    private static final String XML_VERSION = "<?xml version=\"1.0\" encoding=\""
                                            + System.getProperty("file.encoding")
                                            + "\"?>" ;


    // ------------------------------------------------------------- Constructor

    public FolderToXML(TimeZone timezone, String charset) {
        super(timezone, charset);
    }


    // ---------------------------------------------------------- Public Methods

    /**
     * Performs the conversion.
     *
     *
     * @param obj Object. the Mail to be converted in XML
     * @return a string containing the XML representation of this Contact
     * @throws ConverterException
     */
    public String convert(Object obj) throws ConverterException {

        Folder folder = (Folder)obj;

        //StringBuffer output = new StringBuffer(XML_VERSION);
        StringBuffer output = new StringBuffer();
        output.append('<').append(ROOT_TAG).append('>');

        output.append(createTagFromProperty(NAME,     folder.getName()));
        output.append(createTagFromProperty(CREATED,  folder.getCreated()));
        output.append(createTagFromProperty(MODIFIED, folder.getModified()));
        output.append(createTagFromProperty(ACCESSED, folder.getAccessed()));
        output.append(createTagFromProperty(ROLE,     folder.getRole()));

        output.append("</").append(ROOT_TAG).append('>');

        return output.toString();
    }

    //---------------------------------------------------------- PRIVATE METHODS

    /**
     * Returns a string representing the tag with the specified
     * name and the content (value and attributes) fetched from
     * the given Property element
     *
     * @param tag String. the tag name
     * @param prop Property. the property
     * @return StringBuffer. an XML representation of the tag
     * @throws ConverterException
     */
    private StringBuffer createTagFromProperty(String tag, Property prop)
      throws ConverterException {
        StringBuffer output = new StringBuffer("");
        if (prop != null && prop.getPropertyValue() != null) {
            if(!(prop.getPropertyValue().equals(""))) {
                // Create the parameters and use it. NO NEED TO DO THAT 3 times.
                StringBuffer params = getParams(prop);
                if (params.length() > 0) {

                    if (params.indexOf("QUOTED-PRINTABLE") > 0) {
                        output.append('<').append(tag).append(params).append('>')
                              .append(StringTools.escapeBasicXml(
                                         (String)prop.getPropertyValue())
                                     )
                              .append("</").append(tag).append('>');
                    } else {
                        output.append('<').append(tag).append(params).append('>').append(StringTools.escapeBasicXml((String)prop.getPropertyValue())).append("</").append(tag).append('>');
                    }
                } else {
                    output.append('<').append(tag).append('>').append(StringTools.escapeBasicXml((String)prop.getPropertyValue())).append("</").append(tag).append('>');
                }
            } else {
                output.append('<').append(tag).append("/>");
            }
        }
        return output;
    }

    /**
     * Returns a string representing the tag with the specified
     * name and the content (value and attributes) fetched from
     * the given Property element
     *
     * @param tag String. tag the tag name
     * @param propertyValue Object. the property
     * @return StringBuffer. an XML representation of the tag
     */
    /*
    private StringBuffer createTagFromCDATA(String tag, Object propertyValue) {
        StringBuffer output = new StringBuffer();
        if (propertyValue != null) {
            String stringValue = String.valueOf(propertyValue);

            if (stringValue.length()>0) {
                output.append('<').append(tag).append('>');

                output.append("<![CDATA[");
                output.append(stringValue);
                output.append("]]>");
                //output.append("]]&gt;");

                output.append("</").append(tag).append('>');

            } else {
                output.append('<').append(tag).append("/>");
            }
        }
        return output;
    }
    */


    /**
     * Returns a string represnting the attributes fetched from the given Property
     * in an XML-compliant form
     *
     * @param prop the property
     * @return an XML representation of the attributes contained in the Property
     */
    private StringBuffer getParams (Property prop) {
        StringBuffer output = new StringBuffer();

        if (prop.getGroup()!=null) {
            output.append(" GROUP=\"").append(prop.getGroup().toUpperCase()).append("\"");
        }
        if (prop.getEncoding()!=null) {
            output.append(" ENCODING=\"").append(prop.getEncoding().toUpperCase()).append("\"");
        }
        if (prop.getCharset()!=null) {
            output.append(" CHARSET=\"").append(prop.getCharset().toUpperCase()).append("\"");
        }
        if (prop.getLanguage()!=null) {
            output.append(" LANGUAGE=\"").append(prop.getLanguage().toUpperCase()).append("\"");
        }
        if (prop.getValue()!=null) {
            output.append(" VALUE=\"").append(prop.getValue().toUpperCase()).append("\"");
        }
        if (prop.getType()!=null) {
            output.append(" TYPE=\"").append(prop.getType().toUpperCase()).append("\"");
        }
        if (prop.getXParams() != null) {
            HashMap h = prop.getXParams();
            Iterator it = h.keySet().iterator();
            while(it.hasNext()) {
                String tag = (String)it.next();
                String value = (String)h.get(tag);
                output.append(" ").append(tag.toUpperCase()).append("=\"").append(value.toUpperCase()).append("\"");
            }
        }

        return output;
    }

}