/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.util;


import java.util.Date;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;
import java.util.zip.CRC32;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Timestamp;
import javax.mail.Flags;
import javax.mail.Header;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.internet.MimeUtility;
import javax.mail.search.ComparisonTerm;

import com.funambol.email.engine.cache.CrcSyncItemInfo;
import com.funambol.email.exception.EntityException;
import com.funambol.email.items.model.DefaultFolder;
import com.funambol.email.items.model.FlagProperties;
import com.funambol.email.items.model.ItemFolder;
import com.funambol.foundation.common.TimeZonesTools;
import com.funambol.framework.engine.SyncItem;
import com.funambol.framework.engine.SyncItemKey;
import com.funambol.framework.tools.Base64;



/**
 * <p>Utility class</p>
 *
 * @version $Id: Utility.java,v 1.20 2006/05/15 17:34:06 gbmiglia Exp $
 */

public class Utility {

    // ------------------------------------------------------------ Public Methods

    /**
     * credantials format:  "username:passowrd"
     *
     * @param username    String
     * @param credentials String
     * @return password String
     */
    public static String getPassword(String username, String credentials) {
        String up = new String(Base64.decode(credentials));
        int l = username.length();
        return up.substring(l + 1, up.length());
    }


    /**
     * converts the Nokia folder name in the defaul folder name
     *
     * @param oldfolder folder name in the Server String
     * @param df        DefaultFolder
     * @return newfolder folder name in the Nokia device String
     */
    public static String convertNokiaToDefault(String oldfolder, DefaultFolder df) {
        String newfolder = null;
        if (oldfolder.equalsIgnoreCase(Def.NOKIA_INBOX_ENG)) {
            newfolder = df.getInboxName();
        } else if (oldfolder.equalsIgnoreCase(Def.NOKIA_OUTBOX_ENG)) {
            newfolder = df.getOutboxName();
        } else if (oldfolder.equalsIgnoreCase(Def.NOKIA_SENT_ENG)) {
            newfolder = df.getSentName();
        } else if (oldfolder.equalsIgnoreCase(Def.NOKIA_DRAFTS_ENG)) {
            newfolder = df.getDraftName();
        }
        return newfolder;
    }


    /**
     * get content from a SyncItem
     *
     * @param syncItem SyncItem
     * @return content of the SyncItem String
     * @throws EntityException
     */
    public static String getContentFromSyncItem(SyncItem syncItem)
            throws EntityException {
        String content;
        try {
            byte[] itemContent = syncItem.getContent();
            if (itemContent == null) {
                itemContent = new byte[0];
            }
            content = new String(itemContent);
        }
        catch (Exception e) {
            throw new EntityException("Error getting the item " + syncItem, e);
        }

        return content;
    }

   
    /**
     * @param message Message
     * @return String
     */
    public static String getHeaderMessageID(Message message) {
        try {
            String[] values = message.getHeader(Def.HEADER_LABEL_MESSAGE_ID);
            if ((values != null) && (values.length > 0)) {
                return values[0];
            }
        } catch (MessagingException exception) {
            // do nothing; we just want to return null
        }
        return null;
    }


    /**
     * @param message Message
     * @return String
     */
    public static String getHeaderSyncLabel(Message message) {
        try {
            String[] values = message.getHeader(Def.HEADER_LABEL_SYNCMESSAGE);
            if ((values != null) && (values.length > 0)) {
                return values[0];
            }
        } catch (MessagingException exception) {
            // do nothing; we just want to return null
        }
        return null;
    }

    /**
     * Returns an origination date in UTC format
     * Available Date Parser
     * pat_1: Exchange 2003 (eng): wed, 6 Oct 2005 09:46:47 +0200
     * pat_2: Exchange 2000 (eng): 6 Oct 2005 09:46:47 +0200
     * pat_3: BB client          : Wed Dec 28 20:05:53 GMT 2005
     * before apply the pat_4 parser you have to use the
     * private method bb7290toRFC822()
     * pat_4: BB client 7290     : Wed Feb 01 10:49:21 Europe/Amsterdam 2006
     *
     * @param message Message
     * @param loc     Locale
     * @return String
     */
    public static Date getHeaderDate(Message message, Locale loc) {

        //the date inside MS Exchange are in US/UK Locale
        loc = Locale.US;

        DateFormat pat_1 = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z", loc);
        DateFormat pat_2 = new SimpleDateFormat("dd MMM yyyy HH:mm:ss Z", loc);
        DateFormat pat_3 = new SimpleDateFormat("EEE MMM dd HH:mm:ss Z yyyy", loc);
        // remove comment if needed
        //DateFormat pat_4 = new SimpleDateFormat("EEE MMM dd HH:mm:ss z yyyy", loc);

        Date origDate = null;
        String[] values = null;

        try {
            values = message.getHeader("Date");
        } catch (MessagingException me) {
            me.printStackTrace();
        }

        if ((values != null) && (values.length > 0)) {

            String s = values[0];

            // put here the BB7290 convertion if needed

            try {
                origDate = pat_1.parse(s);
            } catch (Exception e1) {
                try {
                    origDate = pat_2.parse(s);
                } catch (Exception e2) {
                    try {
                        origDate = pat_3.parse(s);
                    } catch (Exception e3) {
                        e3.printStackTrace();
                    }
                }
            }
        }

        return origDate;
    }


    /**
     * Date : Thu Oct 06 09:46:47 CEST 2005
     * Returns a received date in UTC format
     *
     * @param message Message
     * @param loc     Locale
     * @return String
     */
    public static String getReceivedDate(Message message, Locale loc) {

        String out = null;

        try {
            Date d = message.getReceivedDate();

            // convert in UTC format
            DateFormat utcf = new SimpleDateFormat("yyyyMMdd'T'HHmmss'Z'", loc);
            utcf.setTimeZone(TimeZone.getTimeZone("UTC"));

            out = utcf.format(d);

        }
        catch (Exception e) {
            // do nothing; we just want to return null
        }
        return out;
    }


    /**
     * the method checks if the folder is a default folder
     * On Exchange server the Default folder are:
     * - /Inbox
     * - /Outbox
     * - /Sent
     * - /Draft
     * - /Trash
     * It's depend of the language (see configuration panel)
     *
     * @param df       DefaultFolder
     * @param fullPath String
     * @return boolean
     * @throws EntityException
     */
    public static boolean defaultFolderChecker(DefaultFolder df, String fullPath)
            throws EntityException {
        return fullPath.equals(Def.SEPARATOR + df.getInboxName()) ||
                fullPath.equals(Def.SEPARATOR + df.getOutboxName()) ||
                fullPath.equals(Def.SEPARATOR + df.getSentName()) ||
                fullPath.equals(Def.SEPARATOR + df.getDraftName()) ||
                fullPath.equals(Def.SEPARATOR + df.getTrashName());
    }

    /**
     * the method checks if the folder is a default folder
     * On Exchange server the Default folder are:
     * - /Inbox
     * - /Outbox
     * - /Sent
     * - /Draft
     * - /Trash
     * It's depend of the language (see configuration panel)
     *
     * @param df       DefaultFolder
     * @param fullPath String
     * @return folder id String
     * @throws EntityException
     */
    public static String getDefaultFolderId(DefaultFolder df, String fullPath)
            throws EntityException {

        if (fullPath.equals(Def.SEPARATOR + df.getInboxName())) {
            return Def.FOLDER_INBOX_GUID;
        }
        if (fullPath.equals(Def.SEPARATOR + df.getOutboxName())) {
            return Def.FOLDER_OUTBOX_GUID;
        }
        if (fullPath.equals(Def.SEPARATOR + df.getSentName())) {
            return Def.FOLDER_SENT_GUID;
        }
        if (fullPath.equals(Def.SEPARATOR + df.getDraftName())) {
            return Def.FOLDER_DRAFTS_GUID;
        }
        if (fullPath.equals(Def.SEPARATOR + df.getTrashName())) {
            return Def.FOLDER_TRASH_GUID;
        }

        throw new EntityException(fullPath + " is not a default folder.");
    }


    /**
     * @param fullpath String
     * @return String
     */
    public static String getFolderNameFromFullpath(String fullpath) {
        String folderName;
        int lastslash = fullpath.lastIndexOf(Def.SEPARATOR);
        folderName = fullpath.substring(lastslash + 1, fullpath.length());
        return folderName;
    }


    /**
     * set folder role using the name
     *
     * @param folderName String
     * @return folder role String
     */
    public static String getFolderRole(String folderName, DefaultFolder df) {
        String folderRole = null;
        if (folderName.equalsIgnoreCase(df.getInboxName())) {
            folderRole = Def.FOLDER_INBOX_ROLE;
        } else if (folderName.equalsIgnoreCase(df.getOutboxName())) {
            folderRole = Def.FOLDER_OUTBOX_ROLE;
        } else if (folderName.equalsIgnoreCase(df.getSentName())) {
            folderRole = Def.FOLDER_SENT_ROLE;
        } else if (folderName.equalsIgnoreCase(df.getDraftName())) {
            folderRole = Def.FOLDER_DRAFTS_ROLE;
        } else if (folderName.equalsIgnoreCase(df.getTrashName())) {
            folderRole = Def.FOLDER_TRASH_ROLE;
        }
        return folderRole;
    }


    /**
     * set creation date of a folder
     *
     * @return UTC creation date String
     */
    public static String getFolderCreationDate() {
        String creationDate;
        creationDate = "20050101T000000Z";
        return creationDate;
    }

    /**
     * gets the parent path of a folder using the full folder path
     *
     * @param fullpath String
     * @return String
     */
    public static String getParentFullpathFromFullpath(String fullpath) {
        String parentFullpath;

        int lastslash = fullpath.lastIndexOf(Def.SEPARATOR);

        // gell all before last slash
        parentFullpath = fullpath.substring(0, lastslash);

        // add initial separator
        parentFullpath = Def.SEPARATOR + parentFullpath;

        if (parentFullpath.equals("/INBOX")) {
            parentFullpath = "/Inbox";
        }

        return parentFullpath;
    }


    /**
     * @param key String
     * @return String
     */
    public static String getIdAfterSeparator(String key) {
        int sepLenght = Def.SEPARATOR.length();
        int endFolderId = key.indexOf(Def.SEPARATOR);
        String idAfter = key;
        if ( endFolderId > 0 ) {
           idAfter = key.substring((endFolderId + sepLenght), key.length());
        }
        return idAfter;
    }


    /**
     * convert the UTC string in a Timestamp
     *
     * @param utc String
     * @return Timestamp
     */
    public static Timestamp UtcToTimestamp(String utc) {

        Timestamp t = null;
        if (utc != null) {
            try {
                DateFormat utcFormatter = new SimpleDateFormat("yyyyMMdd'T'HHmmss'Z'");
                Date d = utcFormatter.parse(utc);
                t = new Timestamp(d.getTime());
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return t;
    }


    /**
     * convert the UTC string in a Date
     *
     * @param utc String
     * @return Date
     */
    public static Date UtcToDate(String utc) {

        Date d = null;

        if (utc != null) {

            try {
                DateFormat utcFormatter = new SimpleDateFormat("yyyyMMdd'T'HHmmss'Z'");
                d = utcFormatter.parse(utc);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return d;
    }

    /**
     * convert Message in a String (RFC2822)
     *
     * @param msg Message
     * @return String
     */
    public static String getStreamFromMessage(Message msg, String encodingType)
            throws MessagingException {
        String streamContentAsString;
        boolean isSeen;
        OutputStream outStream;

        if (msg == null) {
            return null;
        }

        isSeen = msg.isSet(Flags.Flag.SEEN);

        outStream = new ByteArrayOutputStream();


        try {
            if (Def.ENCODE_QUOTED_PRINTABLE.equalsIgnoreCase(encodingType)) {
                msg.writeTo(MimeUtility.encode(outStream, "quoted-printable"));
            } else {
                if (Def.ENCODE_BASE64.equalsIgnoreCase(encodingType)) {
                    msg.writeTo(MimeUtility.encode(outStream, "base64"));
                } else {
                    msg.writeTo(outStream);
                }
            }
        }
        catch (IOException e) {
            throw new MessagingException(e.getMessage(), e);
        }

        streamContentAsString = outStream.toString();

        msg.setFlag(Flags.Flag.SEEN, isSeen);

        return streamContentAsString;
    }

    /**
     * merge arrays
     *
     * @param i CrcSyncItemInfo[]
     * @param o CrcSyncItemInfo[]
     * @param d CrcSyncItemInfo[]
     * @param s CrcSyncItemInfo[]
     * @param t CrcSyncItemInfo[]
     * @return CrcSyncItemInfo[]
     */
    public static String[] mergeArray(String[] i,
                                      String[] o,
                                      String[] d,
                                      String[] s,
                                      String[] t) {

        String[] ids;

        int length = 0;
        int ilength = 0;
        int olength = 0;
        int dlength = 0;
        int slength = 0;
        int tlength = 0;

        if (i != null) {
            ilength = i.length;
            length = ilength;
        }
        if (o != null) {
            olength = o.length;
            length = length + olength;
        }
        if (d != null) {
            dlength = d.length;
            length = length + dlength;
        }
        if (s != null) {
            slength = s.length;
            length = length + slength;
        }
        if (t != null) {
            tlength = t.length;
            length = length + tlength;
        }

        ids = new String[length];

        int yi, yo, yd, ys, yt;

        for (yi = 0; yi < ilength; yi++) {
            ids[yi] = i[yi];
        }
        for (yo = 0; yo < olength; yo++) {
            ids[yi + yo] = o[yo];
        }
        for (yd = 0; yd < dlength; yd++) {
            ids[yi + yo + yd] = d[yd];
        }
        for (ys = 0; ys < slength; ys++) {
            ids[yi + yo + yd + ys] = s[ys];
        }
        for (yt = 0; yt < tlength; yt++) {
            ids[yi + yo + yd + ys + yt] = t[yt];
        }

        return ids;
    }

    /**
     * merge arrays
     *
     * @param i CrcSyncItemInfo[]
     * @param o CrcSyncItemInfo[]
     * @param d CrcSyncItemInfo[]
     * @param s CrcSyncItemInfo[]
     * @param t CrcSyncItemInfo[]
     * @return CrcSyncItemInfo[]
     */
    public static CrcSyncItemInfo[] mergeArray(CrcSyncItemInfo[] i,
                                               CrcSyncItemInfo[] o,
                                               CrcSyncItemInfo[] d,
                                               CrcSyncItemInfo[] s,
                                               CrcSyncItemInfo[] t) {
        CrcSyncItemInfo[] syncItemInfo;

        int length = 0;
        int ilength = 0;
        int olength = 0;
        int dlength = 0;
        int slength = 0;
        int tlength = 0;

        if (i != null) {
            ilength = i.length;
            length = ilength;
        }
        if (o != null) {
            olength = o.length;
            length = length + olength;
        }
        if (d != null) {
            dlength = d.length;
            length = length + dlength;
        }
        if (s != null) {
            slength = s.length;
            length = length + slength;
        }
        if (t != null) {
            tlength = t.length;
            length = length + tlength;
        }

        syncItemInfo = new CrcSyncItemInfo[length];

        int yi;
        int yo;
        int yd;
        int ys;
        int yt;

        for (yi = 0; yi < ilength; yi++) {
            syncItemInfo[yi] = i[yi];
        }
        for (yo = 0; yo < olength; yo++) {
            syncItemInfo[yi + yo] = o[yo];
        }
        for (yd = 0; yd < dlength; yd++) {
            syncItemInfo[yi + yo + yd] = d[yd];
        }
        for (ys = 0; ys < slength; ys++) {
            syncItemInfo[yi + yo + yd + ys] = s[ys];
        }
        for (yt = 0; yt < tlength; yt++) {
            syncItemInfo[yi + yo + yd + ys + yt] = t[yt];
        }

        return syncItemInfo;
    }

    /**
     * merge arrays
     *
     * @param i CrcSyncItemInfo[]
     * @param o CrcSyncItemInfo[]
     * @param d CrcSyncItemInfo[]
     * @param s CrcSyncItemInfo[]
     * @param t CrcSyncItemInfo[]
     * @return CrcSyncItemInfo[]
     */
    public static ItemFolder[] mergeArray(ItemFolder[] i,
                                          ItemFolder[] o,
                                          ItemFolder[] d,
                                          ItemFolder[] s,
                                          ItemFolder[] t) {
        ItemFolder[] itemFolder;

        int length = 0;
        int ilength = 0;
        int olength = 0;
        int dlength = 0;
        int slength = 0;
        int tlength = 0;

        if (i != null) {
            ilength = i.length;
            length = ilength;
        }
        if (o != null) {
            olength = o.length;
            length = length + olength;
        }
        if (d != null) {
            dlength = d.length;
            length = length + dlength;
        }
        if (s != null) {
            slength = s.length;
            length = length + slength;
        }
        if (t != null) {
            tlength = t.length;
            length = length + tlength;
        }

        itemFolder = new ItemFolder[length];

        int yi, yo, yd, ys, yt;

        for (yi = 0; yi < ilength; yi++) {
            itemFolder[yi] = i[yi];
        }
        for (yo = 0; yo < olength; yo++) {
            itemFolder[yi + yo] = o[yo];
        }
        for (yd = 0; yd < dlength; yd++) {
            itemFolder[yi + yo + yd] = d[yd];
        }
        for (ys = 0; ys < slength; ys++) {
            itemFolder[yi + yo + yd + ys] = s[ys];
        }
        for (yt = 0; yt < tlength; yt++) {
            itemFolder[yi + yo + yd + ys + yt] = t[yt];
        }

        return itemFolder;
    }

    /**
     * @param msg Message
     * @return long
     */
    public static long setLastCrc(Message msg) {

        // Compute CRC-32 checksum
        CRC32 checksumEngine = new CRC32();
        long checksumFlag = 0;
        long checksumMessage = 0;

        // set CRC for the Flag properties

        try {

            boolean isSeen = msg.isSet(Flags.Flag.SEEN);
            boolean isAnswered = msg.isSet(Flags.Flag.ANSWERED);
            boolean isDeleted = msg.isSet(Flags.Flag.DELETED);
            boolean isDraft = msg.isSet(Flags.Flag.DRAFT);
            boolean isRecent = msg.isSet(Flags.Flag.RECENT);
            boolean isFlagged = msg.isSet(Flags.Flag.FLAGGED);

            FlagProperties fg = new FlagProperties(isSeen,
                    isAnswered,
                    isDeleted,
                    isDraft,
                    isRecent,
                    isFlagged);

            String fgS = fg.toString();
            byte[] bytesFlags = fgS.getBytes();
            checksumEngine.update(bytesFlags, 0, bytesFlags.length);
            checksumFlag = checksumEngine.getValue();
            checksumEngine.reset();


            String msgS = "";
            Enumeration en = msg.getAllHeaders();
            while (en.hasMoreElements()) {
                Header h = (Header) en.nextElement();
                msgS = msgS + h.getName() + " : " + h.getValue();
            }
            byte[] bytesMsg = msgS.getBytes();
            checksumEngine.update(bytesMsg, 0, bytesMsg.length);
            checksumMessage = checksumEngine.getValue();
            checksumEngine.reset();


        } catch (MessagingException me) {
            me.printStackTrace();
        }

        return checksumFlag + checksumMessage;

    }


    /**
     *
     * set a new header but keep the old Message-ID
     *
     * @param newMsg Message
     * @param oldMsg Message
     * @return String
     * @throws EntityException
     */
    public static String setNewHeader(Message newMsg, Message oldMsg)
            throws EntityException {


        String oldMessageID = null;

        try {

            Header h;
            String name;
            String value;

            Enumeration en = oldMsg.getAllHeaders();
            while (en.hasMoreElements()) {
                h = (Header) en.nextElement();
                name = h.getName();
                value = h.getValue();
                if (name.equalsIgnoreCase(Def.HEADER_LABEL_MESSAGE_ID)) {
                    oldMessageID = value;
                }
                newMsg.setHeader(name, value);
            }

        } catch (MessagingException me) {
            throw new EntityException(me);
        }

        return oldMessageID;

    }


    /**
     * sets the folder target id
     *
     * @param key Sync Item key Value String
     * @return String
     */
    public static String getIdBeforeSeparator(String key) {
        int endFolderId = key.indexOf(Def.SEPARATOR);
        String idBefore = key;
        if (endFolderId > 0){
            idBefore = key.substring(0, endFolderId);
        }
        return idBefore;
    }


    /**
     * the Funambol client send id in format: I/1234
     * the mobile phone can send the LUID not in the format : 1234
     *
     * @param syncItem SyncItem
     * @return FID and FMID in an array String[]
     */
    public static String[] getIdBeforeAndAfter(SyncItem syncItem) {

        String[] keys = new String[2];

        String FMID;
        String folderGUID;
        String FID;
        SyncItemKey parentKey;

        String key = syncItem.getKey().getKeyAsString();

        if (key.indexOf(Def.SEPARATOR) >= 0) {
            //  id format I/1234 (Funambol client ... I get "1234")
            FMID = Utility.getIdAfterSeparator(key);
        } else {
            //  id format 1234 (mobile phone client ... I get "1234")
            FMID = key;
        }

        parentKey = syncItem.getParentKey();

        if (parentKey == null) {
            // get parent from LUID (format I/1234 ... I get "I")
            FID = getIdBeforeSeparator(key);
        } else {
            // get parent from GUID (format ROOT/I ... I get "I")
            folderGUID = parentKey.getKeyAsString();
            FID = getIdAfterSeparator(folderGUID);
        }

        //


        keys[0] = FID;
        keys[1] = FMID;

        return keys;
    }


    /**
     * sets the GUID =  FID + Def.SEPARATOR + FMID
     *
     * @param FID  folder id String
     * @param FMID mail id in the folder String
     * @return String
     */
    public static String getGUID(String FID, String FMID) {
        return FID + Def.SEPARATOR + FMID;
    }


    /**
     *
     * @param timeFilterClause String
     * @return comparison term for time filter int
     */
    public static int getComparisonTerm(String timeFilterClause) {

        int out = ComparisonTerm.GE;

        if (timeFilterClause.equalsIgnoreCase("EQ") ||
                timeFilterClause.equalsIgnoreCase("GT") ||
                timeFilterClause.equalsIgnoreCase("GE")) {
            out = ComparisonTerm.GE;
        } else if (timeFilterClause.equalsIgnoreCase("LT") ||
                timeFilterClause.equalsIgnoreCase("LE")) {
            out = ComparisonTerm.LE;
        }

        return out;
    }

    /**
     * @param origDate Date
     * @return String
     */
    public static String UtcFromDate(Date origDate) {

        String out = null;

        DateFormat utcf = new SimpleDateFormat("yyyyMMdd'T'HHmmss'Z'");

        utcf.setTimeZone(TimeZone.getTimeZone("UTC"));

        if (origDate != null) {
            out = utcf.format(origDate);
        }
        return out;
    }
    
    /**
     * <p>Decode the given <i>text</i> considering it encoded with the given <i>encode type</i>.</p>
     *
     * @param text
     * @param encodeType
     * @return the decoded text
     * @throws MessagingException
     */
    public static String decode(String text, String encodeType)
            throws MessagingException {
        StringBuffer decodedText;
        InputStream inputStream;
        BufferedInputStream buffIn;
        byte[] tempBuff;
        int readLength;

        if (text == null) {
            return null;
        }

        inputStream = new ByteArrayInputStream(text.getBytes());

        if (Def.ENCODE_QUOTED_PRINTABLE.equals(encodeType)) {
            buffIn = new BufferedInputStream(MimeUtility.decode(inputStream, Def.ENCODE_QUOTED_PRINTABLE));
            tempBuff = new byte[1024];
            decodedText = new StringBuffer();
            try {
                while ((readLength = buffIn.read(tempBuff)) > -1) {
                    decodedText.append(new String(tempBuff, 0, readLength));
                }
            }
            catch (IOException e) {
                throw new MessagingException(e.getMessage(), e);
            }
        }

        return text;
    }

    //---------------------------------------------------------- PRIVATE METHODS

 
    
    /**
     * @param dateold String
     * @return String
     * @todo converter for the BB7290 originationDate
     * <p/>
     * Date : Thu Oct 06 09:46:47 CEST 2005
     * Returns a received date in UTC format
     */
    private static String bb7290toRFC822(String dateold) {

        String rfc822 = null;

        int index;
        String label;
        String offset;

        String[] tmzsDescr = TimeZonesTools.getAvailablesTimeZonesDescr();

        for (int i = 0; i < tmzsDescr.length; i++) {

            index = tmzsDescr[i].indexOf("(GMT");

            if (index != -1) {

                label = tmzsDescr[i].substring(0, index - 1);
                offset = tmzsDescr[i].substring(index + 1, tmzsDescr[i].length() - 1);

                index = dateold.indexOf(label);

                if (index != -1) {
                    // is a bb_7290
                    rfc822 = dateold.replaceAll(label, offset);
                } else {
                    //
                    rfc822 = dateold;
                }

            } else {
                // is not a bb_7290
                rfc822 = dateold;
            }

        }

        return rfc822;

    }
}
